@@ -25,19 +25,23 @@
 
 #include <cfloat>
 
+const StringArray AllRADecoderAudioProcessor::weightsStrings = StringArray ("basic", "maxrE", "inphase");
+
 //==============================================================================
 AllRADecoderAudioProcessor::AllRADecoderAudioProcessor()
+: AudioProcessorBase (
 #ifndef JucePlugin_PreferredChannelConfigurations
-: AudioProcessor (BusesProperties()
+                      BusesProperties()
 #if ! JucePlugin_IsMidiEffect
 #if ! JucePlugin_IsSynth
-                  .withInput  ("Input",  AudioChannelSet::discreteChannels(64), true)
+                  .withInput  ("Input",  AudioChannelSet::discreteChannels (64), true)
 #endif
-                  .withOutput ("Output", AudioChannelSet::discreteChannels(64), true)
+                  .withOutput ("Output", AudioChannelSet::discreteChannels (64), true)
 #endif
-                  ),
+                  ,
 #endif
-energyDistribution (Image::PixelFormat::ARGB, 200, 100, true), rEVector (Image::PixelFormat::ARGB, 200, 100, true), oscParams (parameters), parameters (*this, nullptr, "AllRADecoder", createParameterLayout())
+createParameterLayout()),
+energyDistribution (Image::PixelFormat::ARGB, 200, 100, true), rEVector (Image::PixelFormat::ARGB, 200, 100, true)
 {
     // get pointers to the parameters
     inputOrderSetting = parameters.getRawParameterValue ("inputOrderSetting");
@@ -45,6 +49,7 @@ energyDistribution (Image::PixelFormat::ARGB, 200, 100, true), rEVector (Image::
     decoderOrder = parameters.getRawParameterValue ("decoderOrder");
     exportDecoder = parameters.getRawParameterValue ("exportDecoder");
     exportLayout = parameters.getRawParameterValue ("exportLayout");
+    weights = parameters.getRawParameterValue ("weights");
 
     // add listeners to parameter changes
     parameters.addParameterListener ("inputOrderSetting", this);
@@ -57,7 +62,7 @@ energyDistribution (Image::PixelFormat::ARGB, 200, 100, true), rEVector (Image::
     options.folderName          = "IEM";
     options.osxLibrarySubFolder = "Preferences";
 
-    properties = new PropertiesFile(options);
+    properties.reset (new PropertiesFile (options));
     lastDir = File(properties->getValue("presetFolder"));
 
     undoManager.beginNewTransaction();
@@ -88,51 +93,12 @@ energyDistribution (Image::PixelFormat::ARGB, 200, 100, true), rEVector (Image::
 
     loudspeakers.addListener(this);
     prepareLayout();
-
-    oscReceiver.addListener (this);
 }
 
 AllRADecoderAudioProcessor::~AllRADecoderAudioProcessor()
 {
 }
 
-//==============================================================================
-const String AllRADecoderAudioProcessor::getName() const
-{
-    return JucePlugin_Name;
-}
-
-bool AllRADecoderAudioProcessor::acceptsMidi() const
-{
-#if JucePlugin_WantsMidiInput
-    return true;
-#else
-    return false;
-#endif
-}
-
-bool AllRADecoderAudioProcessor::producesMidi() const
-{
-#if JucePlugin_ProducesMidiOutput
-    return true;
-#else
-    return false;
-#endif
-}
-
-bool AllRADecoderAudioProcessor::isMidiEffect() const
-{
-#if JucePlugin_IsMidiEffect
-    return true;
-#else
-    return false;
-#endif
-}
-
-double AllRADecoderAudioProcessor::getTailLengthSeconds() const
-{
-    return 0.0;
-}
 
 int AllRADecoderAudioProcessor::getNumPrograms()
 {
@@ -220,12 +186,6 @@ void AllRADecoderAudioProcessor::releaseResources()
     // spare memory, etc.
 }
 
-#ifndef JucePlugin_PreferredChannelConfigurations
-bool AllRADecoderAudioProcessor::isBusesLayoutSupported (const BusesLayout& layouts) const
-{
-    return true;
-}
-#endif
 
 void AllRADecoderAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBuffer& midiMessages)
 {
@@ -253,11 +213,11 @@ void AllRADecoderAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBu
         const int L = buffer.getNumSamples();
         AudioBlock<float> inputAudioBlock = AudioBlock<float>(buffer.getArrayOfWritePointers(), nChIn, L);
         AudioBlock<float> outputAudioBlock = AudioBlock<float>(buffer.getArrayOfWritePointers(), nChOut, L);
-        ProcessContextNonReplacing<float> decoderContext (inputAudioBlock, outputAudioBlock);
-        decoder.process(decoderContext);
 
-        for (int ch = nChOut; ch < nChIn; ++ch) // clear all not needed channels
-            buffer.clear(ch, 0, buffer.getNumSamples());
+        decoder.process (inputAudioBlock, outputAudioBlock);
+
+        for (int ch = nChOut; ch < buffer.getNumChannels(); ++ch) // clear all not needed channels
+            buffer.clear (ch, 0, buffer.getNumSamples());
 
     }
     noiseBurst.processBuffer (buffer);
@@ -281,11 +241,13 @@ void AllRADecoderAudioProcessor::getStateInformation (MemoryBlock& destData)
     {
         parameters.state.removeChild(parameters.state.getChildWithName("Loudspeakers"), nullptr);
     }
-    parameters.state.appendChild(loudspeakers, nullptr);
+    parameters.state.appendChild (loudspeakers, nullptr);
 
-    parameters.state.setProperty ("OSCPort", var(oscReceiver.getPortNumber()), nullptr);
+    auto state = parameters.copyState();
+    auto oscConfig = state.getOrCreateChildWithName ("OSCConfig", nullptr);
+    oscConfig.copyPropertiesFrom (oscParameterInterface.getConfig(), nullptr);
 
-    ScopedPointer<XmlElement> xml (parameters.state.createXml());
+    std::unique_ptr<XmlElement> xml (state.createXml());
     copyXmlToBinary (*xml, destData);
 }
 
@@ -293,16 +255,21 @@ void AllRADecoderAudioProcessor::getStateInformation (MemoryBlock& destData)
 
 void AllRADecoderAudioProcessor::setStateInformation (const void* data, int sizeInBytes)
 {
-    ScopedPointer<XmlElement> xmlState (getXmlFromBinary (data, sizeInBytes));
+    std::unique_ptr<XmlElement> xmlState (getXmlFromBinary (data, sizeInBytes));
     if (xmlState != nullptr)
     {
         if (xmlState->hasTagName (parameters.state.getType()))
         {
             parameters.replaceState (ValueTree::fromXml (*xmlState));
-            if (parameters.state.hasProperty ("OSCPort"))
+            if (parameters.state.hasProperty ("OSCPort")) // legacy
             {
-                oscReceiver.connect (parameters.state.getProperty ("OSCPort", var(-1)));
+                oscParameterInterface.getOSCReceiver().connect (parameters.state.getProperty ("OSCPort", var (-1)));
+                parameters.state.removeProperty ("OSCPort", nullptr);
             }
+
+            auto oscConfig = parameters.state.getChildWithName ("OSCConfig");
+            if (oscConfig.isValid())
+                oscParameterInterface.setConfig (oscConfig);
         }
 
         XmlElement* lsps (xmlState->getChildByName("Loudspeakers"));
@@ -696,6 +663,7 @@ Result AllRADecoderAudioProcessor::calculateDecoder()
         return Result::fail("Layout not ready!");
 
     const int N = roundToInt(*decoderOrder) + 1;
+    const auto ambisonicWeights = ReferenceCountedDecoder::Weights (roundToInt (*weights));
     const int nCoeffs = square(N+1);
     const int nLsps = (int) points.size();
     const int nRealLsps = nLsps - imaginaryFlags.countNumberOfSetBits();
@@ -842,9 +810,10 @@ Result AllRADecoderAudioProcessor::calculateDecoder()
     const float wHalf = w / 2;
     const int h = energyDistribution.getHeight();
     const float hHalf = h / 2;
-    float maxSumOfSquares = 0.0f;
     float minLvl = 0.0f;
     float maxLvl = 0.0f;
+    float sumLvl = 0.0f;
+    auto levelValues = Matrix<float> (w, h);
     for (int y = 0; y < h; ++y)
         for (int x = 0; x < w; ++x)
         {
@@ -853,6 +822,12 @@ Result AllRADecoderAudioProcessor::calculateDecoder()
             Vector3D<float> cart = sphericalInRadiansToCartesian(spher);
             SHEval(N, cart.x, cart.y, cart.z, &sh[0]); // encoding a source
 
+            if (ambisonicWeights == ReferenceCountedDecoder::Weights::maxrE)
+                multiplyMaxRE (N, sh.data());
+            else if (ambisonicWeights == ReferenceCountedDecoder::Weights::inPhase)
+                multiplyInPhase (N, sh.data());
+
+
             Vector3D<float> rE (0.0f, 0.0f, 0.0f);
             float sumOfSquares = 0.0f;
             for (int m = 0; m < nRealLsps; ++m)
@@ -861,53 +836,48 @@ Result AllRADecoderAudioProcessor::calculateDecoder()
                 for (int n = 0; n < nCoeffs; ++n)
                     sum += (sh[n] * decoderMatrix(m, n));
                 const float sumSquared = square(sum);
-                rE = rE + realLspsCoordinates[m] * sumSquared;
+                rE += realLspsCoordinates[m] * sumSquared;
                 sumOfSquares += sumSquared;
             }
 
-            rE /= sumOfSquares + FLT_EPSILON;
-            const float width = 2.0f * acos(jmin(1.0f, rE.length()));
+            const float lvl = 0.5f * Decibels::gainToDecibels (sumOfSquares);
+            levelValues(x, y) = lvl;
+            sumLvl += lvl;
 
-            const float lvl = 0.5f * Decibels::gainToDecibels(sumOfSquares);
             if (lvl > maxLvl)
                 maxLvl = lvl;
             if (lvl < minLvl)
                 minLvl = lvl;
-            const float map = jlimit(-0.5f, 0.5f, 0.5f * 0.16666667f * Decibels::gainToDecibels(sumOfSquares)) + 0.5f;
 
-            const float reMap = jlimit(0.0f, 1.0f, width / (float) MathConstants<float>::pi);
+            rE /= sumOfSquares + FLT_EPSILON;
+            const float width = 2.0f * acos(jmin(1.0f, rE.length()));
+            const float reMap = jlimit (0.0f, 1.0f, width / MathConstants<float>::pi);
 
-            Colour rEPixelColour = Colours::limegreen.withMultipliedAlpha(reMap);
-            Colour pixelColour = Colours::red.withMultipliedAlpha(map);
-
-            rEVector.setPixelAt(x, y, rEPixelColour);
-            energyDistribution.setPixelAt(x, y, pixelColour);
-
-            if (sumOfSquares > maxSumOfSquares)
-                maxSumOfSquares = sumOfSquares;
+            const Colour rEPixelColour = Colours::limegreen.withMultipliedAlpha(reMap);
+            rEVector.setPixelAt (x, y, rEPixelColour);
         }
 
+    const float meanLvl = sumLvl / (w * h);
+    for (int y = 0; y < h; ++y)
+        for (int x = 0; x < w; ++x)
+        {
+            constexpr float plusMinusRange = 1.5f;
+            const float map = (jlimit (-plusMinusRange, plusMinusRange, levelValues(x, y) - meanLvl) + plusMinusRange) / (2 * plusMinusRange);
+
+            const Colour pixelColour = Colours::red.withMultipliedAlpha (map);
+            energyDistribution.setPixelAt (x, y, pixelColour);
+        }
 
     DBG("min: " << minLvl << " max: " << maxLvl);
 
     updateLoudspeakerVisualization = true;
 
-    Colour colormapData[8];
-    colormapData[0] = Colours::skyblue.withMultipliedAlpha(0.0f);
-    colormapData[1] = Colours::skyblue.withMultipliedAlpha(0.2f);
-    colormapData[2] = Colours::skyblue.withMultipliedAlpha(0.3f);
-    colormapData[3] = Colour::fromFloatRGBA(0.167f, 0.620f, 0.077f, 6.0f);
-    colormapData[4] = Colour::fromFloatRGBA(0.167f, 0.620f, 0.077f, 7.0f);
-    colormapData[5] = Colour::fromFloatRGBA(0.8f, 0.620f, 0.077f, 0.8f);
-    colormapData[6] = Colour::fromFloatRGBA(0.8f, 0.620f, 0.077f, 1.0f);
-    colormapData[7] = Colours::red;
-
 
     ReferenceCountedDecoder::Ptr newDecoder = new ReferenceCountedDecoder("Decoder", "A " + getOrderString(N) + " order Ambisonics decoder using the AllRAD approach.", (int) decoderMatrix.getSize()[0], (int) decoderMatrix.getSize()[1]);
     newDecoder->getMatrix() = decoderMatrix;
     ReferenceCountedDecoder::Settings newSettings;
     newSettings.expectedNormalization = ReferenceCountedDecoder::Normalization::n3d;
-    newSettings.weights = ReferenceCountedDecoder::Weights::maxrE;
+    newSettings.weights = ambisonicWeights;
     newSettings.weightsAlreadyApplied = false;
 
     newDecoder->setSettings(newSettings);
@@ -1018,7 +988,7 @@ void AllRADecoderAudioProcessor::saveConfigurationToFile (File destination)
         DBG("Configuration successfully written to file.");
         MailBox::Message newMessage;
         newMessage.messageColour = Colours::green;
-        newMessage.headline = "Configuration export successfully";
+        newMessage.headline = "Configuration exported successfully";
         newMessage.text = "The decoder was successfully written to " + destination.getFileName() + ".";
         messageToEditor = newMessage;
         updateMessage = true;
@@ -1107,67 +1077,59 @@ void AllRADecoderAudioProcessor::rotate (const float degreesAddedToAzimuth)
     updateTable = true;
 }
 
-//==============================================================================
-pointer_sized_int AllRADecoderAudioProcessor::handleVstPluginCanDo (int32 index,
-                                                                    pointer_sized_int value, void* ptr, float opt)
-{
-    auto text = (const char*) ptr;
-    auto matches = [=](const char* s) { return strcmp (text, s) == 0; };
-
-    if (matches ("wantsChannelCountNotifications"))
-        return 1;
-    return 0;
-}
 
 //==============================================================================
-void AllRADecoderAudioProcessor::oscMessageReceived (const OSCMessage &message)
+const bool AllRADecoderAudioProcessor::interceptOSCMessage (OSCMessage &message)
 {
-    String prefix ("/" + String(JucePlugin_Name));
-    if (! message.getAddressPattern().toString().startsWith (prefix))
-        return;
-
-    OSCMessage msg (message);
-    msg.setAddressPattern (message.getAddressPattern().toString().substring(String(JucePlugin_Name).length() + 1));
-
-    if (msg.getAddressPattern().toString().equalsIgnoreCase("/decoderOrder") && msg.size() >= 1)
+    if (message.getAddressPattern().toString().equalsIgnoreCase ("/" + String (JucePlugin_Name) + "/decoderOrder") && message.size() >= 1)
     {
-        if (msg[0].isInt32())
+        if (message[0].isInt32())
         {
-            auto value = msg[0].getInt32() - 1;
-            msg.clear();
-            msg.addInt32 (value);
+            auto value = message[0].getInt32() - 1;
+            message.clear();
+            message.addInt32 (value);
         }
-        else if (msg[0].isFloat32())
+        else if (message[0].isFloat32())
         {
-            auto value = msg[0].getFloat32() - 1;
-            msg.clear();
-            msg.addFloat32 (value);
+            auto value = message[0].getFloat32() - 1;
+            message.clear();
+            message.addFloat32 (value);
         }
-
     }
 
-    if (! oscParams.processOSCMessage (msg))
+    return false;
+}
+
+const bool AllRADecoderAudioProcessor::processNotYetConsumedOSCMessage (const OSCMessage &message)
+{
+    const String prefix ("/" + String (JucePlugin_Name));
+    if (message.getAddressPattern().toString().startsWith (prefix))
     {
-        // Load configuration file
-        if (msg.getAddressPattern().toString().equalsIgnoreCase("/loadFile") && msg.size() >= 1)
+        OSCMessage msg (message);
+        msg.setAddressPattern (message.getAddressPattern().toString().substring (String (JucePlugin_Name).length() + 1));
+
+        if (msg.getAddressPattern().toString().equalsIgnoreCase ("/loadFile") && msg.size() >= 1)
         {
             if (msg[0].isString())
             {
                 File fileToLoad (msg[0].getString());
                 loadConfiguration (fileToLoad);
+                return true;
             }
         }
-        else if (msg.getAddressPattern().toString().equalsIgnoreCase("/calculate") ||
-                 msg.getAddressPattern().toString().equalsIgnoreCase("/calculateDecoder"))
+        else if (msg.getAddressPattern().toString().equalsIgnoreCase ("/calculate") ||
+                 msg.getAddressPattern().toString().equalsIgnoreCase ("/calculateDecoder"))
         {
             calculateDecoder();
+            return true;
         }
-        else if (msg.getAddressPattern().toString().equalsIgnoreCase("/export") && msg.size() >= 1)
+        else if (msg.getAddressPattern().toString().equalsIgnoreCase ("/export") && msg.size() >= 1)
         {
             if (msg[0].isString())
             {
                 File file (msg[0].getString());
                 saveConfigurationToFile (file);
+                return true;
             }
         }
         else if (msg.getAddressPattern().toString().equalsIgnoreCase ("/playNoise") && msg.size() >= 1)
@@ -1176,7 +1138,10 @@ void AllRADecoderAudioProcessor::oscMessageReceived (const OSCMessage &message)
             {
                 const int channel = msg[0].getInt32();
                 if (channel <= 64)
+                {
                     playNoiseBurst (channel);
+                    return true;
+                }
             }
         }
         else if (msg.getAddressPattern().toString().equalsIgnoreCase ("/playEncodedNoise") && msg.size() >= 2)
@@ -1189,39 +1154,31 @@ void AllRADecoderAudioProcessor::oscMessageReceived (const OSCMessage &message)
             else if (msg[0].isFloat32())
                 azimuth = msg[0].getFloat32();
             else
-                return;
+                return false;
 
             if (msg[1].isInt32())
                 elevation = msg[1].getInt32();
             else if (msg[1].isFloat32())
                 elevation = msg[1].getFloat32();
             else
-                return;
+                return false;
 
             playAmbisonicNoiseBurst (azimuth, elevation);
+            return true;
         }
     }
+
+    return false;
 }
 
-void AllRADecoderAudioProcessor::oscBundleReceived (const OSCBundle &bundle)
-{
-    for (int i = 0; i < bundle.size(); ++i)
-    {
-        auto elem = bundle[i];
-        if (elem.isMessage())
-            oscMessageReceived (elem.getMessage());
-        else if (elem.isBundle())
-            oscBundleReceived (elem.getBundle());
-    }
-}
 
 // ==============================================================================
-AudioProcessorValueTreeState::ParameterLayout AllRADecoderAudioProcessor::createParameterLayout()
+std::vector<std::unique_ptr<RangedAudioParameter>> AllRADecoderAudioProcessor::createParameterLayout()
 {
     // add your audio parameters here
     std::vector<std::unique_ptr<RangedAudioParameter>> params;
 
-    params.push_back (oscParams.createAndAddParameter ("inputOrderSetting", "Input Ambisonic Order", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("inputOrderSetting", "Input Ambisonic Order", "",
                                      NormalisableRange<float> (0.0f, 8.0f, 1.0f), 0.0f,
                                      [](float value) {
                                          if (value >= 0.5f && value < 1.5f) return "0th";
@@ -1235,14 +1192,14 @@ AudioProcessorValueTreeState::ParameterLayout AllRADecoderAudioProcessor::create
                                          else return "Auto";},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("useSN3D", "Input Normalization", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("useSN3D", "Input Normalization", "",
                                     NormalisableRange<float>(0.0f, 1.0f, 1.0f), 1.0f,
                                     [](float value) {
                                         if (value >= 0.5f) return "SN3D";
                                         else return "N3D";
                                     }, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("decoderOrder", "Decoder Order", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("decoderOrder", "Decoder Order", "",
                                      NormalisableRange<float> (0.0f, 6.0f, 1.0f), 0.0f,
                                      [](float value) {
                                          if (value >= 0.5f && value < 1.5f) return "2nd";
@@ -1254,20 +1211,21 @@ AudioProcessorValueTreeState::ParameterLayout AllRADecoderAudioProcessor::create
                                          else return "1st";},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("exportDecoder", "Export Decoder", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("exportDecoder", "Export Decoder", "",
                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
                                      [](float value) {
                                          if (value >= 0.5f) return "Yes";
                                          else return "No";
                                      }, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("exportLayout", "Export Layout", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("exportLayout", "Export Layout", "",
                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
                                      [](float value) {
                                          if (value >= 0.5f) return "Yes";
                                          else return "No";
                                      }, nullptr));
 
+    params.push_back (std::make_unique<AudioParameterChoice> ("weights", "Ambisonic Weights", weightsStrings, 1));
 
-    return { params.begin(), params.end() };
+    return params;
 }
