@@ -23,29 +23,17 @@
 #pragma once
 
 #include "../JuceLibraryCode/JuceHeader.h"
-#include "../../resources/IOHelper.h"
+#include "../../resources/AudioProcessorBase.h"
 #include "../../resources/Conversions.h"
 
-// ===== OSC ====
-#include "../../resources/OSCParameterInterface.h"
-#include "../../resources/OSCReceiverPlus.h"
-
+#define ProcessorClass CoordinateConverterAudioProcessor
 
 //==============================================================================
-/**
- Use the IOHelper to detect which amount of channels or which Ambisonic order is possible with the current bus layout.
- The available IOTypes are:
-    - AudioChannels<maxChannelCount>
-    - Ambisonics<maxOrder> (can also be used for directivity signals)
- You can leave `maxChannelCount` and `maxOrder` empty for default values (64 channels and 7th order)
-*/
-class CoordinateConverterAudioProcessor  : public AudioProcessor,
-                                        public AudioProcessorValueTreeState::Listener,
-                                        public IOHelper<IOTypes::AudioChannels<64>, IOTypes::AudioChannels<64>>,
-                                        public VSTCallbackHandler,
-                                        private OSCReceiver::Listener<OSCReceiver::RealtimeCallback>
+class CoordinateConverterAudioProcessor  : public AudioProcessorBase<IOTypes::AudioChannels<64>, IOTypes::AudioChannels<64>>
 {
 public:
+    constexpr static int numberOfInputChannels = 64;
+    constexpr static int numberOfOutputChannels = 64;
     //==============================================================================
     CoordinateConverterAudioProcessor();
     ~CoordinateConverterAudioProcessor();
@@ -54,24 +42,12 @@ public:
     void prepareToPlay (double sampleRate, int samplesPerBlock) override;
     void releaseResources() override;
 
-   #ifndef JucePlugin_PreferredChannelConfigurations
-    bool isBusesLayoutSupported (const BusesLayout& layouts) const override;
-   #endif
-
     void processBlock (AudioSampleBuffer&, MidiBuffer&) override;
 
     //==============================================================================
     AudioProcessorEditor* createEditor() override;
     bool hasEditor() const override;
 
-    //==============================================================================
-    const String getName() const override;
-
-    bool acceptsMidi() const override;
-    bool producesMidi() const override;
-    bool isMidiEffect () const override;
-    double getTailLengthSeconds() const override;
-
     //==============================================================================
     int getNumPrograms() override;
     int getCurrentProgram() override;
@@ -87,22 +63,9 @@ public:
     void parameterChanged (const String &parameterID, float newValue) override;
     void updateBuffers() override; // use this to implement a buffer update method
 
-    //======== PluginCanDo =========================================================
-    pointer_sized_int handleVstManufacturerSpecific (int32 index, pointer_sized_int value,
-                                                     void* ptr, float opt) override { return 0; };
-
-    pointer_sized_int handleVstPluginCanDo (int32 index, pointer_sized_int value,
-                                            void* ptr, float opt) override;
-    //==============================================================================
-
-    //======== OSC =================================================================
-    void oscMessageReceived (const OSCMessage &message) override;
-    void oscBundleReceived (const OSCBundle &bundle) override;
-    OSCReceiverPlus& getOSCReceiver () { return oscReceiver; }
-    //==============================================================================
 
     //======= Parameters ===========================================================
-    AudioProcessorValueTreeState::ParameterLayout createParameterLayout();
+    std::vector<std::unique_ptr<RangedAudioParameter>> createParameterLayout();
     //==============================================================================
 
     void updateCartesianCoordinates();
@@ -113,11 +76,7 @@ public:
     Atomic<bool> repaintPositionPlanes = true;
 
 private:
-    // ====== parameters
-    OSCParameterInterface oscParams;
-    OSCReceiverPlus oscReceiver;
-    AudioProcessorValueTreeState parameters;
-    
+    //==============================================================================
     Atomic<bool> updatingParams = false;
     bool cartesianWasLastUpdated = true;
 
