@@ -25,27 +25,21 @@
 
 #include "../JuceLibraryCode/JuceHeader.h"
 #include "../hammerAitovSample.h"
-#include <Eigen/Dense>
 #include "../../resources/efficientSHvanilla.h"
 #include "../../resources/ambisonicTools.h"
-#include "../../resources/IOHelper.h"
+#include "../../resources/AudioProcessorBase.h"
 #include "../../resources/MaxRE.h"
 
-// ===== OSC ====
-#include "../../resources/OSCParameterInterface.h"
-#include "../../resources/OSCReceiverPlus.h"
+#define ProcessorClass EnergyVisualizerAudioProcessor
 
 //==============================================================================
 /**
 */
-class EnergyVisualizerAudioProcessor  : public AudioProcessor,
-                                        public AudioProcessorValueTreeState::Listener,
-                                        public IOHelper<IOTypes::Ambisonics<>, IOTypes::Nothing>,
-                                        public VSTCallbackHandler,
-                                        private OSCReceiver::Listener<OSCReceiver::RealtimeCallback>,
-                                        private Timer
+class EnergyVisualizerAudioProcessor  : public AudioProcessorBase<IOTypes::Ambisonics<>, IOTypes::Nothing>, private Timer
 {
 public:
+    constexpr static int numberOfInputChannels = 64;
+    constexpr static int numberOfOutputChannels = 64;
     //==============================================================================
     EnergyVisualizerAudioProcessor();
     ~EnergyVisualizerAudioProcessor();
@@ -54,24 +48,12 @@ public:
     void prepareToPlay (double sampleRate, int samplesPerBlock) override;
     void releaseResources() override;
 
-   #ifndef JucePlugin_PreferredChannelConfigurations
-    bool isBusesLayoutSupported (const BusesLayout& layouts) const override;
-   #endif
-
     void processBlock (AudioSampleBuffer&, MidiBuffer&) override;
 
     //==============================================================================
     AudioProcessorEditor* createEditor() override;
     bool hasEditor() const override;
 
-    //==============================================================================
-    const String getName() const override;
-
-    bool acceptsMidi() const override;
-    bool producesMidi() const override;
-    bool isMidiEffect () const override;
-    double getTailLengthSeconds() const override;
-
     //==============================================================================
     int getNumPrograms() override;
     int getCurrentProgram() override;
@@ -86,44 +68,32 @@ public:
     //==============================================================================
     void parameterChanged (const String &parameterID, float newValue) override;
 
-    //======== PluginCanDo =========================================================
-    pointer_sized_int handleVstManufacturerSpecific (int32 index, pointer_sized_int value,
-                                                     void* ptr, float opt) override { return 0; };
-    pointer_sized_int handleVstPluginCanDo (int32 index, pointer_sized_int value,
-                                            void* ptr, float opt) override;
-    //==============================================================================
-
-    //======== OSC =================================================================
-    void oscMessageReceived (const OSCMessage &message) override;
-    void oscBundleReceived (const OSCBundle &bundle) override;
-    OSCReceiverPlus& getOSCReceiver () { return oscReceiver; }
-    //==============================================================================
 
     //======= Parameters ===========================================================
-    AudioProcessorValueTreeState::ParameterLayout createParameterLayout();
+    std::vector<std::unique_ptr<RangedAudioParameter>> createParameterLayout();
     //==============================================================================
 
-    Array<float> rms;
+    const float getPeakLevelSetting() { return *peakLevel; }
+    const float getDynamicRange() { return *dynamicRange; }
+
+    std::vector<float> rms;
     Atomic<Time> lastEditorTime;
 
 private:
-    OSCParameterInterface oscParams;
-    OSCReceiverPlus oscReceiver;
-    AudioProcessorValueTreeState parameters;
-
-    Eigen::DiagonalMatrix<float, 64> maxReWeights;
-
-    float timeConstant;
-    // parameter
+    //==============================================================================
+    // parameters
     float *orderSetting, *useSN3D, *peakLevel, *dynamicRange;
 
+    float timeConstant;
+
     Atomic<bool> doProcessing = true;
-    
-    Eigen::Matrix<float,nSamplePoints,64,Eigen::ColMajor> YH;
-    Eigen::Matrix<float,nSamplePoints,64,Eigen::ColMajor> workingMatrix;
-    AudioSampleBuffer sampledSignals;
+
+    dsp::Matrix<float> decoderMatrix;
+    std::vector<float> weights;
+    std::vector<float> sampledSignal;
 
     void timerCallback() override;
+    void sendAdditionalOSCMessages (OSCSender& oscSender, const OSCAddressPattern& address) override;
 
     //==============================================================================
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (EnergyVisualizerAudioProcessor)
