@@ -25,19 +25,19 @@
 
 //==============================================================================
 FdnReverbAudioProcessor::FdnReverbAudioProcessor()
-
+: AudioProcessorBase (
 #ifndef JucePlugin_PreferredChannelConfigurations
-:AudioProcessor (BusesProperties()
+                  BusesProperties()
 #if ! JucePlugin_IsMidiEffect
 #if ! JucePlugin_IsSynth
                  .withInput  ("Input",  AudioChannelSet::discreteChannels(64), true)
 #endif
                  .withOutput ("Output", AudioChannelSet::discreteChannels(64), true)
 #endif
-                 ),
+                 ,
 #endif
 
-oscParams (parameters), parameters (*this, nullptr, "FdnReverb", createParameterLayout())
+createParameterLayout())
 {
     parameters.addParameterListener ("delayLength", this);
     parameters.addParameterListener ("revTime", this);
@@ -65,8 +65,6 @@ oscParams (parameters), parameters (*this, nullptr, "FdnReverb", createParameter
     fdn.setFdnSize(FeedbackDelayNetwork::big);
 	fdnFade.setFdnSize(FeedbackDelayNetwork::big);
 	fdnFade.setDryWet(1.0f);
-
-    oscReceiver.addListener (this);
 }
 
 FdnReverbAudioProcessor::~FdnReverbAudioProcessor()
@@ -74,34 +72,6 @@ FdnReverbAudioProcessor::~FdnReverbAudioProcessor()
 }
 
 //==============================================================================
-const String FdnReverbAudioProcessor::getName() const
-{
-    return JucePlugin_Name;
-}
-
-bool FdnReverbAudioProcessor::acceptsMidi() const
-{
-   #if JucePlugin_WantsMidiInput
-    return true;
-   #else
-    return false;
-   #endif
-}
-
-bool FdnReverbAudioProcessor::producesMidi() const
-{
-   #if JucePlugin_ProducesMidiOutput
-    return true;
-   #else
-    return false;
-   #endif
-}
-
-double FdnReverbAudioProcessor::getTailLengthSeconds() const
-{
-    return 10.0;
-}
-
 int FdnReverbAudioProcessor::getNumPrograms()
 {
     return 1;   // NB: some hosts don't cope very well if you tell them there are 0 programs,
@@ -213,20 +183,12 @@ void FdnReverbAudioProcessor::reset()
 	fdnFade.reset();
 }
 
-//------------------------------------------------------------------------------
-#ifndef JucePlugin_PreferredChannelConfigurations
-bool FdnReverbAudioProcessor::isBusesLayoutSupported (const BusesLayout& layouts) const
-{
-    return true;
-}
-#endif
-
 //------------------------------------------------------------------------------
 void FdnReverbAudioProcessor::processBlock (AudioBuffer<float>& buffer, MidiBuffer& midiMessages)
 {
 	const int nChannels = buffer.getNumChannels();
 	const int nSamples = buffer.getNumSamples();
-	
+
 	// make copy of input data
 	if (*fadeInTime != 0.0f)
 	{
@@ -285,7 +247,10 @@ AudioProcessorEditor* FdnReverbAudioProcessor::createEditor()
 void FdnReverbAudioProcessor::getStateInformation (MemoryBlock &destData)
 {
     auto state = parameters.copyState();
-    state.setProperty ("OSCPort", var(oscReceiver.getPortNumber()), nullptr);
+
+    auto oscConfig = state.getOrCreateChildWithName ("OSCConfig", nullptr);
+    oscConfig.copyPropertiesFrom (oscParameterInterface.getConfig(), nullptr);
+
     std::unique_ptr<XmlElement> xml (state.createXml());
     copyXmlToBinary (*xml, destData);
 }
@@ -297,111 +262,79 @@ void FdnReverbAudioProcessor::setStateInformation (const void *data, int sizeInB
         if (xmlState->hasTagName (parameters.state.getType()))
         {
             parameters.replaceState (ValueTree::fromXml (*xmlState));
-            if (parameters.state.hasProperty ("OSCPort"))
+            if (parameters.state.hasProperty ("OSCPort")) // legacy
             {
-                oscReceiver.connect (parameters.state.getProperty ("OSCPort", var (-1)));
+                oscParameterInterface.getOSCReceiver().connect (parameters.state.getProperty ("OSCPort", var (-1)));
+                parameters.state.removeProperty ("OSCPort", nullptr);
             }
+
+            auto oscConfig = parameters.state.getChildWithName ("OSCConfig");
+            if (oscConfig.isValid())
+                oscParameterInterface.setConfig (oscConfig);
         }
 }
 
 
 //==============================================================================
-pointer_sized_int FdnReverbAudioProcessor::handleVstPluginCanDo (int32 index,
-                                                                     pointer_sized_int value, void* ptr, float opt)
-{
-    auto text = (const char*) ptr;
-    auto matches = [=](const char* s) { return strcmp (text, s) == 0; };
-
-    if (matches ("wantsChannelCountNotifications"))
-        return 1;
-    return 0;
-}
-
-//==============================================================================
-void FdnReverbAudioProcessor::oscMessageReceived (const OSCMessage &message)
-{
-    String prefix ("/" + String(JucePlugin_Name));
-    if (! message.getAddressPattern().toString().startsWith (prefix))
-        return;
-
-    OSCMessage msg (message);
-    msg.setAddressPattern (message.getAddressPattern().toString().substring(String(JucePlugin_Name).length() + 1));
-
-    oscParams.processOSCMessage (msg);
-}
-
-void FdnReverbAudioProcessor::oscBundleReceived (const OSCBundle &bundle)
-{
-    for (int i = 0; i < bundle.size(); ++i)
-    {
-        auto elem = bundle[i];
-        if (elem.isMessage())
-            oscMessageReceived (elem.getMessage());
-        else if (elem.isBundle())
-            oscBundleReceived (elem.getBundle());
-    }
-}
-
-//==============================================================================
-AudioProcessorValueTreeState::ParameterLayout FdnReverbAudioProcessor::createParameterLayout()
+std::vector<std::unique_ptr<RangedAudioParameter>> FdnReverbAudioProcessor::createParameterLayout()
 {
     // add your audio parameters here
     std::vector<std::unique_ptr<RangedAudioParameter>> params;
 
 
-    params.push_back (oscParams.createAndAddParameter ("delayLength", "Room Size", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("delayLength", "Room Size", "",
                                      NormalisableRange<float> (1.0f, 30.0f, 1.0f), 20.0f,
                                      [](float value) {return String (value, 0);},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("revTime", "Reverberation Time", "s",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("revTime", "Reverberation Time", "s",
                                      NormalisableRange<float> (0.1f, 9.0f, 0.1f), 5.f,
                                      [](float value) {return String (value, 1);},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("lowCutoff", "Lows Cutoff Frequency", "Hz",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("lowCutoff", "Lows Cutoff Frequency", "Hz",
                                      NormalisableRange<float> (20.f, 20000.f, 1.f, 0.2f), 100.f,
                                      [](float value) {return String (value, 0);},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("lowQ", "Lows Q Factor", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("lowQ", "Lows Q Factor", "",
                                      NormalisableRange<float> (0.01f, 0.9f, 0.01f), 0.5f,
                                      [](float value) {return String (value, 2);},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("lowGain",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("lowGain",
                                      "Lows Gain", "dB/s",
                                      NormalisableRange<float> (-80.0f, 6.0, 0.1f), 1.f,
                                      [](float value) {return String (value, 1);},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("highCutoff", "Highs Cutoff Frequency", "Hz",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("highCutoff", "Highs Cutoff Frequency", "Hz",
                                      NormalisableRange<float> (20.f, 20000.f, 1.f, 0.2f), 2000.f,
                                      [](float value) {return String (value, 0);},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("highQ", "Highs Q Factor", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("highQ", "Highs Q Factor", "",
                                      NormalisableRange<float> (0.01f, 0.9f, 0.01f), 0.5f,
                                      [](float value) {return String (value, 2);},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("highGain",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("highGain",
                                      "Highs Gain", "dB/s",
                                      NormalisableRange<float> (-80.0f, 4.0f, 0.1f), -10.f,
                                      [](float value) {return String (value, 1);},
                                      nullptr));
-    
-    params.push_back (oscParams.createAndAddParameter ("dryWet", "Dry/Wet", "",
+
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("dryWet", "Dry/Wet", "",
                                      NormalisableRange<float> (0.f, 1.f, 0.01f), 0.5f,
                                      [](float value) {return String (value, 2);},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("fadeInTime", "Fdn Time", "s",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("fadeInTime", "Fade-in Time", "s",
                                      NormalisableRange<float> (0.0f, 9.0f, 0.01f), 0.f,
                                      [](float value) {return String(value, 2);},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("fdnSize", "Fdn Size", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("fdnSize", "Fdn Size (internal)", "",
                                      NormalisableRange<float> (0.0f, 2.0f, 1.0f), 2.0f,
                                      [](float value) {
                                          if (value == 0.0f)
@@ -413,8 +346,7 @@ AudioProcessorValueTreeState::ParameterLayout FdnReverbAudioProcessor::createPar
                                      },
                                      nullptr));
 
-
-    return { params.begin(), params.end() };
+    return params;
 }
 
 //==============================================================================
