@@ -24,22 +24,23 @@
 #include "PluginEditor.h"
 
 
-static constexpr int filterTypePresets[] = {1, 1, 1, 1, 1, 1};
-static constexpr float filterFrequencyPresets[] = {80.0f, 120.0f, 1600.0f, 2200.0f, 8000.0f, 16000.0f};
+static constexpr int filterTypePresets[] = {1, 1, 1, 1, 1, 3};
+static constexpr float filterFrequencyPresets[] = {20.0f, 120.0f, 500.0f, 2200.0f, 8000.0f, 16000.0f};
 
 //==============================================================================
 MultiEQAudioProcessor::MultiEQAudioProcessor()
+     : AudioProcessorBase (
 #ifndef JucePlugin_PreferredChannelConfigurations
-     : AudioProcessor (BusesProperties()
+                           BusesProperties()
                      #if ! JucePlugin_IsMidiEffect
                       #if ! JucePlugin_IsSynth
                        .withInput  ("Input",  AudioChannelSet::discreteChannels (64), true)
                       #endif
                        .withOutput ("Output", AudioChannelSet::discreteChannels (64), true)
                      #endif
-                       ),
+                       ,
 #endif
-oscParams (parameters), parameters (*this, nullptr, "MultiEQ", createParameterLayout())
+createParameterLayout())
 {
     // get pointers to the parameters
     inputChannelsSetting = parameters.getRawParameterValue ("inputChannelsSetting");
@@ -93,9 +94,6 @@ oscParams (parameters), parameters (*this, nullptr, "MultiEQ", createParameterLa
     additionalFilterArrays[1].clear();
     for (int ch = 0; ch < ceil (64 / IIRfloat_elements()); ++ch)
         additionalFilterArrays[1].add (new IIR::Filter<IIRfloat> (additionalProcessorCoefficients[1]));
-
-
-    oscReceiver.addListener (this);
 }
 
 
@@ -106,29 +104,30 @@ MultiEQAudioProcessor::~MultiEQAudioProcessor()
 
 void MultiEQAudioProcessor::updateGuiCoefficients()
 {
-    const double sampleRate = getSampleRate();
+    const double sampleRate = getSampleRate() == 0 ? 48000.0 : getSampleRate();
 
     // Low band
+    const auto lowBandFrequency = jmin (static_cast<float> (0.5 * sampleRate), *filterFrequency[0]);
     const SpecialFilterType lowType = SpecialFilterType (static_cast<int> (*filterType[0]));
 
     switch (lowType)
     {
         case SpecialFilterType::LinkwitzRileyHighPass:
         {
-            auto coeffs = IIR::Coefficients<double>::makeHighPass (sampleRate, *filterFrequency[0]);
+            auto coeffs = IIR::Coefficients<double>::makeHighPass (sampleRate, lowBandFrequency);
             coeffs->coefficients = FilterVisualizerHelper<double>::cascadeSecondOrderCoefficients
             (coeffs->coefficients, coeffs->coefficients);
             guiCoefficients[0] = coeffs;
             break;
         }
         case SpecialFilterType::FirstOrderHighPass:
-            guiCoefficients[0] = IIR::Coefficients<double>::makeFirstOrderHighPass (sampleRate, *filterFrequency[0]);
+            guiCoefficients[0] = IIR::Coefficients<double>::makeFirstOrderHighPass (sampleRate, lowBandFrequency);
             break;
         case SpecialFilterType::SecondOrderHighPass:
-            guiCoefficients[0] = IIR::Coefficients<double>::makeHighPass (sampleRate, *filterFrequency[0], *filterQ[0]);
+            guiCoefficients[0] = IIR::Coefficients<double>::makeHighPass (sampleRate, lowBandFrequency, *filterQ[0]);
             break;
         case SpecialFilterType::LowShelf:
-            guiCoefficients[0] = IIR::Coefficients<double>::makeLowShelf (sampleRate, *filterFrequency[0], *filterQ[0], Decibels::decibelsToGain (*filterGain[0]));
+            guiCoefficients[0] = IIR::Coefficients<double>::makeLowShelf (sampleRate, lowBandFrequency, *filterQ[0], Decibels::decibelsToGain (*filterGain[0]));
             break;
         default:
             break;
@@ -136,26 +135,27 @@ void MultiEQAudioProcessor::updateGuiCoefficients()
 
 
     // High band
+    const auto highBandFrequency = jmin (static_cast<float> (0.5 * sampleRate), *filterFrequency[numFilterBands - 1]);
     const SpecialFilterType highType = SpecialFilterType (4 + static_cast<int> (*filterType[numFilterBands - 1]));
 
     switch (highType)
     {
         case SpecialFilterType::LinkwitzRileyLowPass:
         {
-            auto coeffs = IIR::Coefficients<double>::makeLowPass (sampleRate, *filterFrequency[numFilterBands - 1]);
+            auto coeffs = IIR::Coefficients<double>::makeLowPass (sampleRate, highBandFrequency);
             coeffs->coefficients = FilterVisualizerHelper<double>::cascadeSecondOrderCoefficients
             (coeffs->coefficients, coeffs->coefficients);
             guiCoefficients[numFilterBands - 1] = coeffs;
             break;
         }
         case SpecialFilterType::FirstOrderLowPass:
-            guiCoefficients[numFilterBands - 1] = IIR::Coefficients<double>::makeFirstOrderLowPass (sampleRate, *filterFrequency[numFilterBands - 1]);
+            guiCoefficients[numFilterBands - 1] = IIR::Coefficients<double>::makeFirstOrderLowPass (sampleRate, highBandFrequency);
             break;
         case SpecialFilterType::SecondOrderLowPass:
-            guiCoefficients[numFilterBands - 1] = IIR::Coefficients<double>::makeLowPass (sampleRate, *filterFrequency[numFilterBands - 1], *filterQ[numFilterBands - 1]);
+            guiCoefficients[numFilterBands - 1] = IIR::Coefficients<double>::makeLowPass (sampleRate, highBandFrequency, *filterQ[numFilterBands - 1]);
             break;
         case SpecialFilterType::HighShelf:
-            guiCoefficients[numFilterBands - 1] = IIR::Coefficients<double>::makeHighShelf (sampleRate, *filterFrequency[numFilterBands - 1], *filterQ[numFilterBands - 1], Decibels::decibelsToGain (*filterGain[numFilterBands - 1]));
+            guiCoefficients[numFilterBands - 1] = IIR::Coefficients<double>::makeHighShelf (sampleRate, highBandFrequency, *filterQ[numFilterBands - 1], Decibels::decibelsToGain (*filterGain[numFilterBands - 1]));
             break;
         default:
             break;
@@ -165,17 +165,18 @@ void MultiEQAudioProcessor::updateGuiCoefficients()
 
     for (int f = 1; f < numFilterBands - 1; ++f)
     {
+        const auto frequency = jmin (static_cast<float> (0.5 * sampleRate), *filterFrequency[f]);
         const RegularFilterType type = RegularFilterType (2 + static_cast<int>(*filterType[f]));
         switch (type)
         {
             case RegularFilterType::LowShelf:
-                guiCoefficients[f] = IIR::Coefficients<double>::makeLowShelf (sampleRate, *filterFrequency[f], *filterQ[f], Decibels::decibelsToGain (*filterGain[f]));
+                guiCoefficients[f] = IIR::Coefficients<double>::makeLowShelf (sampleRate, frequency, *filterQ[f], Decibels::decibelsToGain (*filterGain[f]));
                 break;
             case RegularFilterType::PeakFilter:
-                guiCoefficients[f] = IIR::Coefficients<double>::makePeakFilter (sampleRate, *filterFrequency[f], *filterQ[f], Decibels::decibelsToGain (*filterGain[f]));
+                guiCoefficients[f] = IIR::Coefficients<double>::makePeakFilter (sampleRate, frequency, *filterQ[f], Decibels::decibelsToGain (*filterGain[f]));
                 break;
             case RegularFilterType::HighShelf:
-                guiCoefficients[f] = IIR::Coefficients<double>::makeHighShelf (sampleRate, *filterFrequency[f], *filterQ[f], Decibels::decibelsToGain (*filterGain[f]));
+                guiCoefficients[f] = IIR::Coefficients<double>::makeHighShelf (sampleRate, frequency, *filterQ[f], Decibels::decibelsToGain (*filterGain[f]));
                 break;
             default:
                 break;
@@ -186,62 +187,64 @@ void MultiEQAudioProcessor::updateGuiCoefficients()
 
 inline dsp::IIR::Coefficients<float>::Ptr MultiEQAudioProcessor::createFilterCoefficients (const RegularFilterType type, const double sampleRate, const float frequency, const float Q, const float gain)
 {
+    const auto f = jmin (static_cast<float> (0.5 * sampleRate), frequency);
     switch (type)
     {
         case RegularFilterType::FirstOrderHighPass:
-            return IIR::Coefficients<float>::makeFirstOrderHighPass (sampleRate, frequency);
+            return IIR::Coefficients<float>::makeFirstOrderHighPass (sampleRate, f);
             break;
         case RegularFilterType::SecondOrderHighPass:
-            return IIR::Coefficients<float>::makeHighPass (sampleRate, frequency, Q);
+            return IIR::Coefficients<float>::makeHighPass (sampleRate, f, Q);
             break;
         case RegularFilterType::LowShelf:
-            return IIR::Coefficients<float>::makeLowShelf (sampleRate, frequency, Q, gain);
+            return IIR::Coefficients<float>::makeLowShelf (sampleRate, f, Q, gain);
             break;
         case RegularFilterType::PeakFilter:
-            return IIR::Coefficients<float>::makePeakFilter (sampleRate, frequency, Q, gain);
+            return IIR::Coefficients<float>::makePeakFilter (sampleRate, f, Q, gain);
             break;
         case RegularFilterType::HighShelf:
-            return IIR::Coefficients<float>::makeHighShelf (sampleRate, frequency, Q, gain);
+            return IIR::Coefficients<float>::makeHighShelf (sampleRate, f, Q, gain);
             break;
         case RegularFilterType::FirstOrderLowPass:
-            return IIR::Coefficients<float>::makeFirstOrderLowPass (sampleRate, frequency);
+            return IIR::Coefficients<float>::makeFirstOrderLowPass (sampleRate, f);
             break;
         case RegularFilterType::SecondOrderLowPass:
-            return IIR::Coefficients<float>::makeLowPass (sampleRate, frequency, Q);
+            return IIR::Coefficients<float>::makeLowPass (sampleRate, f, Q);
             break;
         default:
-            return IIR::Coefficients<float>::makeAllPass (sampleRate, frequency, Q);
+            return IIR::Coefficients<float>::makeAllPass (sampleRate, f, Q);
             break;
     }
 }
 
 inline dsp::IIR::Coefficients<double>::Ptr MultiEQAudioProcessor::createFilterCoefficientsForGui (const RegularFilterType type, const double sampleRate, const float frequency, const float Q, const float gain)
 {
+    const auto f = jmin (static_cast<float> (0.5 * sampleRate), frequency);
     switch (type)
     {
         case RegularFilterType::FirstOrderHighPass:
-            return IIR::Coefficients<double>::makeFirstOrderHighPass (sampleRate, frequency);
+            return IIR::Coefficients<double>::makeFirstOrderHighPass (sampleRate, f);
             break;
         case RegularFilterType::SecondOrderHighPass:
-            return IIR::Coefficients<double>::makeHighPass (sampleRate, frequency, Q);
+            return IIR::Coefficients<double>::makeHighPass (sampleRate, f, Q);
             break;
         case RegularFilterType::LowShelf:
-            return IIR::Coefficients<double>::makeLowShelf (sampleRate, frequency, Q, gain);
+            return IIR::Coefficients<double>::makeLowShelf (sampleRate, f, Q, gain);
             break;
         case RegularFilterType::PeakFilter:
-            return IIR::Coefficients<double>::makePeakFilter (sampleRate, frequency, Q, gain);
+            return IIR::Coefficients<double>::makePeakFilter (sampleRate, f, Q, gain);
             break;
         case RegularFilterType::HighShelf:
-            return IIR::Coefficients<double>::makeHighShelf (sampleRate, frequency, Q, gain);
+            return IIR::Coefficients<double>::makeHighShelf (sampleRate, f, Q, gain);
             break;
         case RegularFilterType::FirstOrderLowPass:
-            return IIR::Coefficients<double>::makeFirstOrderLowPass (sampleRate, frequency);
+            return IIR::Coefficients<double>::makeFirstOrderLowPass (sampleRate, f);
             break;
         case RegularFilterType::SecondOrderLowPass:
-            return IIR::Coefficients<double>::makeLowPass (sampleRate, frequency, Q);
+            return IIR::Coefficients<double>::makeLowPass (sampleRate, f, Q);
             break;
         default:
-            return IIR::Coefficients<double>::makeAllPass (sampleRate, frequency, Q);
+            return IIR::Coefficients<double>::makeAllPass (sampleRate, f, Q);
             break;
     }
 }
@@ -250,12 +253,14 @@ void MultiEQAudioProcessor::createLinkwitzRileyFilter (const bool isUpperBand)
 {
     if (isUpperBand)
     {
-        tempCoefficients[numFilterBands - 1] = IIR::Coefficients<float>::makeLowPass (getSampleRate(), *filterFrequency[numFilterBands - 1], *filterQ[numFilterBands - 1]);
+        const auto frequency = jmin (static_cast<float> (0.5 * getSampleRate()), *filterFrequency[numFilterBands - 1]);
+        tempCoefficients[numFilterBands - 1] = IIR::Coefficients<float>::makeLowPass (getSampleRate(), frequency, *filterQ[numFilterBands - 1]);
         additionalTempCoefficients[1] = processorCoefficients[numFilterBands - 1];
     }
     else
     {
-        tempCoefficients[0] = IIR::Coefficients<float>::makeHighPass (getSampleRate(), *filterFrequency[0], *filterQ[0]);
+        const auto frequency = jmin (static_cast<float> (0.5 * getSampleRate()), *filterFrequency[0]);
+        tempCoefficients[0] = IIR::Coefficients<float>::makeHighPass (getSampleRate(), frequency, *filterQ[0]);
         additionalTempCoefficients[0] = processorCoefficients[0];
     }
 }
@@ -297,7 +302,7 @@ void MultiEQAudioProcessor::createFilterCoefficients (const int filterIndex, con
                 break;
 
             default:
-                jassert (type < 2);
+                jassert (type <= 2);
                 switch (type)
                 {
                     case 0:
@@ -326,49 +331,21 @@ void MultiEQAudioProcessor::copyFilterCoefficientsToProcessor()
 
     *additionalProcessorCoefficients[0] = *additionalTempCoefficients[0];
     *additionalProcessorCoefficients[1] = *additionalTempCoefficients[1];
-    
+
     userHasChangedFilterSettings = false;
 }
 
 
+inline void MultiEQAudioProcessor::clear (AudioBlock<IIRfloat>& ab)
+{
+    const int N = static_cast<int> (ab.getNumSamples()) * IIRfloat_elements();
+    const int nCh = static_cast<int> (ab.getNumChannels());
+
+    for (int ch = 0; ch < nCh; ++ch)
+        FloatVectorOperations::clear (reinterpret_cast<float*> (ab.getChannelPointer (ch)), N);
+}
+
 //==============================================================================
-const String MultiEQAudioProcessor::getName() const
-{
-    return JucePlugin_Name;
-}
-
-bool MultiEQAudioProcessor::acceptsMidi() const
-{
-   #if JucePlugin_WantsMidiInput
-    return true;
-   #else
-    return false;
-   #endif
-}
-
-bool MultiEQAudioProcessor::producesMidi() const
-{
-   #if JucePlugin_ProducesMidiOutput
-    return true;
-   #else
-    return false;
-   #endif
-}
-
-bool MultiEQAudioProcessor::isMidiEffect() const
-{
-   #if JucePlugin_IsMidiEffect
-    return true;
-   #else
-    return false;
-   #endif
-}
-
-double MultiEQAudioProcessor::getTailLengthSeconds() const
-{
-    return 0.0;
-}
-
 int MultiEQAudioProcessor::getNumPrograms()
 {
     return 1;   // NB: some hosts don't cope very well if you tell them there are 0 programs,
@@ -416,7 +393,8 @@ void MultiEQAudioProcessor::prepareToPlay (double sampleRate, int samplesPerBloc
         }
 
         interleavedData.add (new AudioBlock<IIRfloat> (interleavedBlockData[i], 1, samplesPerBlock));
-        interleavedData.getLast()->clear();
+        //interleavedData.getLast()->clear(); // this one's broken in JUCE 5.4.5
+        clear (*interleavedData.getLast());
     }
 
     zero = AudioBlock<float> (zeroData, IIRfloat_elements(), samplesPerBlock);
@@ -429,12 +407,6 @@ void MultiEQAudioProcessor::releaseResources()
     // spare memory, etc.
 }
 
-#ifndef JucePlugin_PreferredChannelConfigurations
-bool MultiEQAudioProcessor::isBusesLayoutSupported (const BusesLayout& layouts) const
-{
-    return true;
-}
-#endif
 
 void MultiEQAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBuffer& midiMessages)
 {
@@ -497,7 +469,9 @@ void MultiEQAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBuffer&
         {
             for (int i = 0; i < nSIMDFilters; ++i)
             {
-                ProcessContextReplacing<IIRfloat> context (*interleavedData[i]);
+                const SIMDRegister<float>* chPtr[1] = {interleavedData[i]->getChannelPointer (0)};
+                AudioBlock<SIMDRegister<float>> ab (const_cast<SIMDRegister<float>**> (chPtr), 1, L);
+                ProcessContextReplacing<SIMDRegister<float>> context (ab);
                 filterArrays[f][i]->process (context);
             }
         }
@@ -508,7 +482,9 @@ void MultiEQAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBuffer&
     {
         for (int i = 0; i < nSIMDFilters; ++i)
         {
-            ProcessContextReplacing<IIRfloat> context (*interleavedData[i]);
+            const SIMDRegister<float>* chPtr[1] = {chPtr[0] = interleavedData[i]->getChannelPointer (0)};
+            AudioBlock<SIMDRegister<float>> ab (const_cast<SIMDRegister<float>**> (chPtr), 1, L);
+            ProcessContextReplacing<SIMDRegister<float>> context (ab);
             additionalFilterArrays[0][i]->process (context);
         }
     }
@@ -516,7 +492,9 @@ void MultiEQAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBuffer&
     {
         for (int i = 0; i < nSIMDFilters; ++i)
         {
-            ProcessContextReplacing<IIRfloat> context (*interleavedData[i]);
+            const SIMDRegister<float>* chPtr[1] = {interleavedData[i]->getChannelPointer (0)};
+            AudioBlock<SIMDRegister<float>> ab (const_cast<SIMDRegister<float>**> (chPtr), 1, L);
+            ProcessContextReplacing<SIMDRegister<float>> context (ab);
             additionalFilterArrays[1][i]->process (context);
         }
     }
@@ -577,29 +555,32 @@ AudioProcessorEditor* MultiEQAudioProcessor::createEditor()
 //==============================================================================
 void MultiEQAudioProcessor::getStateInformation (MemoryBlock& destData)
 {
-    // You should use this method to store your parameters in the memory block.
-    // You could do that either as raw data, or use the XML or ValueTree classes
-    // as intermediaries to make it easy to save and load complex data.
-    auto state = parameters.copyState();
-    state.setProperty ("OSCPort", var (oscReceiver.getPortNumber()), nullptr);
-    std::unique_ptr<XmlElement> xml (state.createXml());
-    copyXmlToBinary (*xml, destData);
+  auto state = parameters.copyState();
+
+  auto oscConfig = state.getOrCreateChildWithName ("OSCConfig", nullptr);
+  oscConfig.copyPropertiesFrom (oscParameterInterface.getConfig(), nullptr);
+
+  std::unique_ptr<XmlElement> xml (state.createXml());
+  copyXmlToBinary (*xml, destData);
 }
 
 
 void MultiEQAudioProcessor::setStateInformation (const void* data, int sizeInBytes)
 {
-    // You should use this method to restore your parameters from this memory block,
-    // whose contents will have been created by the getStateInformation() call.
     std::unique_ptr<XmlElement> xmlState (getXmlFromBinary (data, sizeInBytes));
     if (xmlState.get() != nullptr)
         if (xmlState->hasTagName (parameters.state.getType()))
         {
             parameters.replaceState (ValueTree::fromXml (*xmlState));
-            if (parameters.state.hasProperty ("OSCPort"))
+            if (parameters.state.hasProperty ("OSCPort")) // legacy
             {
-                oscReceiver.connect (parameters.state.getProperty ("OSCPort", var (-1)));
+                oscParameterInterface.getOSCReceiver().connect (parameters.state.getProperty ("OSCPort", var (-1)));
+                parameters.state.removeProperty ("OSCPort", nullptr);
             }
+
+            auto oscConfig = parameters.state.getChildWithName ("OSCConfig");
+            if (oscConfig.isValid())
+                oscParameterInterface.setConfig (oscConfig);
         }
 }
 
@@ -628,61 +609,24 @@ void MultiEQAudioProcessor::updateBuffers()
 }
 
 //==============================================================================
-pointer_sized_int MultiEQAudioProcessor::handleVstPluginCanDo (int32 index,
-                                                                     pointer_sized_int value, void* ptr, float opt)
-{
-    auto text = (const char*) ptr;
-    auto matches = [=](const char* s) { return strcmp (text, s) == 0; };
-
-    if (matches ("wantsChannelCountNotifications"))
-        return 1;
-    return 0;
-}
-
-//==============================================================================
-void MultiEQAudioProcessor::oscMessageReceived (const OSCMessage &message)
-{
-    String prefix ("/" + String (JucePlugin_Name));
-    if (! message.getAddressPattern().toString().startsWith (prefix))
-        return;
-
-    OSCMessage msg (message);
-    msg.setAddressPattern (message.getAddressPattern().toString().substring (String (JucePlugin_Name).length() + 1));
-
-    oscParams.processOSCMessage (msg);
-}
-
-void MultiEQAudioProcessor::oscBundleReceived (const OSCBundle &bundle)
-{
-    for (int i = 0; i < bundle.size(); ++i)
-    {
-        auto elem = bundle[i];
-        if (elem.isMessage())
-            oscMessageReceived (elem.getMessage());
-        else if (elem.isBundle())
-            oscBundleReceived (elem.getBundle());
-    }
-}
-
-//==============================================================================
-AudioProcessorValueTreeState::ParameterLayout MultiEQAudioProcessor::createParameterLayout()
+std::vector<std::unique_ptr<RangedAudioParameter>> MultiEQAudioProcessor::createParameterLayout()
 {
     // add your audio parameters here
     std::vector<std::unique_ptr<RangedAudioParameter>> params;
 
 
 
-    params.push_back (oscParams.createAndAddParameter ("inputChannelsSetting", "Number of input channels ", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("inputChannelsSetting", "Number of input channels ", "",
                                      NormalisableRange<float> (0.0f, 64.0f, 1.0f), 0.0f,
                                      [](float value) {return value < 0.5f ? "Auto" : String (value);}, nullptr));
 
 
     int i = 0;
-    params.push_back (oscParams.createAndAddParameter ("filterEnabled" + String (i), "Filter Enablement " + String (i + 1), "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("filterEnabled" + String (i), "Filter Enablement " + String (i + 1), "",
                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
                                      [](float value) { return value < 0.5 ? String ("OFF") : String ("ON");}, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("filterType" + String (i), "Filter Type " + String (i + 1), "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("filterType" + String (i), "Filter Type " + String (i + 1), "",
                                      NormalisableRange<float> (0.0f, 3.0f, 1.0f),  filterTypePresets[i],
                                      [](float value) {
                                          if (value < 0.5f) return "HP (6dB/oct)";
@@ -691,16 +635,16 @@ AudioProcessorValueTreeState::ParameterLayout MultiEQAudioProcessor::createParam
                                          else return "Low-shelf";},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("filterFrequency" + String (i), "Filter Frequency " + String (i + 1), "Hz",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("filterFrequency" + String (i), "Filter Frequency " + String (i + 1), "Hz",
                                      NormalisableRange<float> (20.0f, 20000.0f, 1.0f, 0.4f), filterFrequencyPresets[i],
                                      [](float value) { return String(value, 0); }, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("filterQ" + String (i), "Filter Q " + String (i+1), "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("filterQ" + String (i), "Filter Q " + String (i+1), "",
                                      NormalisableRange<float> (0.05f, 8.0f, 0.05f), 0.7f,
                                      [](float value) { return String (value, 2); },
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("filterGain" + String (i), "Filter Gain " + String (i + 1), "dB",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("filterGain" + String (i), "Filter Gain " + String (i + 1), "dB",
                                      NormalisableRange<float> (-60.0f, 15.0f, 0.1f), 0.0f,
                                      [](float value) { return String (value, 1); },
                                      nullptr));
@@ -708,11 +652,11 @@ AudioProcessorValueTreeState::ParameterLayout MultiEQAudioProcessor::createParam
 
     for (int i = 1; i < numFilterBands - 1; ++i)
     {
-        params.push_back (oscParams.createAndAddParameter ("filterEnabled" + String (i), "Filter Enablement " + String (i + 1), "",
+        params.push_back (OSCParameterInterface::createParameterTheOldWay ("filterEnabled" + String (i), "Filter Enablement " + String (i + 1), "",
                                          NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
                                          [](float value) { return value < 0.5 ? String ("OFF") : String ("ON");}, nullptr));
 
-        params.push_back (oscParams.createAndAddParameter ("filterType" + String (i), "Filter Type " + String (i + 1), "",
+        params.push_back (OSCParameterInterface::createParameterTheOldWay ("filterType" + String (i), "Filter Type " + String (i + 1), "",
                                          NormalisableRange<float> (0.0f, 2.0f, 1.0f),  filterTypePresets[i],
                                          [](float value) {
                                              if (value < 0.5f) return "Low-shelf";
@@ -720,16 +664,16 @@ AudioProcessorValueTreeState::ParameterLayout MultiEQAudioProcessor::createParam
                                              else return "High-shelf";},
                                          nullptr));
 
-        params.push_back (oscParams.createAndAddParameter ("filterFrequency" + String (i), "Filter Frequency " + String (i + 1), "Hz",
+        params.push_back (OSCParameterInterface::createParameterTheOldWay ("filterFrequency" + String (i), "Filter Frequency " + String (i + 1), "Hz",
                                          NormalisableRange<float> (20.0f, 20000.0f, 1.0f, 0.4f), filterFrequencyPresets[i],
                                          [](float value) { return String(value, 0); }, nullptr));
 
-        params.push_back (oscParams.createAndAddParameter ("filterQ" + String (i), "Filter Q " + String (i+1), "",
+        params.push_back (OSCParameterInterface::createParameterTheOldWay ("filterQ" + String (i), "Filter Q " + String (i+1), "",
                                          NormalisableRange<float> (0.05f, 8.0f, 0.05f), 0.7f,
                                          [](float value) { return String (value, 2); },
                                          nullptr));
 
-        params.push_back (oscParams.createAndAddParameter ("filterGain" + String (i), "Filter Gain " + String (i + 1), "dB",
+        params.push_back (OSCParameterInterface::createParameterTheOldWay ("filterGain" + String (i), "Filter Gain " + String (i + 1), "dB",
                                          NormalisableRange<float> (-60.0f, 15.0f, 0.1f), 0.0f,
                                          [](float value) { return String (value, 1); },
                                          nullptr));
@@ -737,11 +681,11 @@ AudioProcessorValueTreeState::ParameterLayout MultiEQAudioProcessor::createParam
 
     i = numFilterBands - 1;
 
-    params.push_back (oscParams.createAndAddParameter ("filterEnabled" + String (i), "Filter Enablement " + String (i + 1), "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("filterEnabled" + String (i), "Filter Enablement " + String (i + 1), "",
                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
                                      [](float value) { return value < 0.5 ? String ("OFF") : String ("ON");}, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("filterType" + String (i), "Filter Type " + String (i + 1), "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("filterType" + String (i), "Filter Type " + String (i + 1), "",
                                      NormalisableRange<float> (0.0f, 3.0f, 1.0f),  filterTypePresets[i],
                                      [](float value) {
                                          if (value < 0.5f) return "LP (6dB/Oct)";
@@ -750,24 +694,22 @@ AudioProcessorValueTreeState::ParameterLayout MultiEQAudioProcessor::createParam
                                          else return "High-shelf";},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("filterFrequency" + String (i), "Filter Frequency " + String (i + 1), "Hz",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("filterFrequency" + String (i), "Filter Frequency " + String (i + 1), "Hz",
                                      NormalisableRange<float> (20.0f, 20000.0f, 1.0f, 0.4f), filterFrequencyPresets[i],
                                      [](float value) { return String(value, 0); }, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("filterQ" + String (i), "Filter Q " + String (i+1), "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("filterQ" + String (i), "Filter Q " + String (i+1), "",
                                      NormalisableRange<float> (0.05f, 8.0f, 0.05f), 0.7f,
                                      [](float value) { return String (value, 2); },
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("filterGain" + String (i), "Filter Gain " + String (i + 1), "dB",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("filterGain" + String (i), "Filter Gain " + String (i + 1), "dB",
                                      NormalisableRange<float> (-60.0f, 15.0f, 0.1f), 0.0f,
                                      [](float value) { return String (value, 1); },
                                      nullptr));
 
 
-
-
-    return { params.begin(), params.end() };
+    return params;
 }
 
 //==============================================================================
