@@ -26,19 +26,30 @@
 
 //==============================================================================
 MultiEncoderAudioProcessor::MultiEncoderAudioProcessor()
-
+: AudioProcessorBase (
 #ifndef JucePlugin_PreferredChannelConfigurations
-:AudioProcessor (BusesProperties()
+                 BusesProperties()
 #if ! JucePlugin_IsMidiEffect
 #if ! JucePlugin_IsSynth
                  .withInput  ("Input",  AudioChannelSet::discreteChannels(maxNumberOfInputs), true)
 #endif
                  .withOutput ("Output", AudioChannelSet::discreteChannels(64), true)
 #endif
-                 ),
+                 ,
 #endif
-oscParams (parameters), parameters (*this, nullptr, "MultiEncoder", createParameterLayout())
+createParameterLayout())
 {
+    // global properties
+    PropertiesFile::Options options;
+    options.applicationName     = "MultiEncoder";
+    options.filenameSuffix      = "settings";
+    options.folderName          = "IEM";
+    options.osxLibrarySubFolder = "Preferences";
+
+    properties.reset (new PropertiesFile (options));
+    lastDir = File (properties->getValue ("presetFolder"));
+
+
     parameters.addParameterListener("masterAzimuth", this);
     parameters.addParameterListener("masterElevation", this);
     parameters.addParameterListener("masterRoll", this);
@@ -50,8 +61,6 @@ oscParams (parameters), parameters (*this, nullptr, "MultiEncoder", createParame
     muteMask.clear();
     soloMask.clear();
 
-
-
     for (int i = 0; i < maxNumberOfInputs; ++i)
     {
         azimuth[i] = parameters.getRawParameterValue ("azimuth"+String(i));
@@ -92,42 +101,12 @@ oscParams (parameters), parameters (*this, nullptr, "MultiEncoder", createParame
     }
 
     updateQuaternions();
-
-    oscReceiver.addListener (this);
 }
 
 MultiEncoderAudioProcessor::~MultiEncoderAudioProcessor()
 {
 }
 
-//==============================================================================
-const String MultiEncoderAudioProcessor::getName() const
-{
-    return JucePlugin_Name;
-}
-
-bool MultiEncoderAudioProcessor::acceptsMidi() const
-{
-#if JucePlugin_WantsMidiInput
-    return true;
-#else
-    return false;
-#endif
-}
-
-bool MultiEncoderAudioProcessor::producesMidi() const
-{
-#if JucePlugin_ProducesMidiOutput
-    return true;
-#else
-    return false;
-#endif
-}
-
-double MultiEncoderAudioProcessor::getTailLengthSeconds() const
-{
-    return 0.0;
-}
 
 int MultiEncoderAudioProcessor::getNumPrograms()
 {
@@ -165,13 +144,6 @@ void MultiEncoderAudioProcessor::releaseResources()
     // spare memory, etc.
 }
 
-#ifndef JucePlugin_PreferredChannelConfigurations
-bool MultiEncoderAudioProcessor::isBusesLayoutSupported (const BusesLayout& layouts) const
-{
-    return true;
-}
-#endif
-
 void MultiEncoderAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBuffer& midiMessages)
 {
     checkInputAndOutput(this, *inputSetting, *orderSetting);
@@ -330,16 +302,20 @@ void MultiEncoderAudioProcessor::parameterChanged (const String &parameterID, fl
 //==============================================================================
 void MultiEncoderAudioProcessor::getStateInformation (MemoryBlock& destData)
 {
+    auto state = parameters.copyState();
     for (int i = 0; i < maxNumberOfInputs; ++i)
-        parameters.state.setProperty("colour" + String(i), elementColours[i].toString(), nullptr);
-    parameters.state.setProperty ("OSCPort", var(oscReceiver.getPortNumber()), nullptr);
-    ScopedPointer<XmlElement> xml (parameters.state.createXml());
+        state.setProperty ("colour" + String(i), elementColours[i].toString(), nullptr);
+
+    auto oscConfig = state.getOrCreateChildWithName ("OSCConfig", nullptr);
+    oscConfig.copyPropertiesFrom (oscParameterInterface.getConfig(), nullptr);
+
+    std::unique_ptr<XmlElement> xml (state.createXml());
     copyXmlToBinary (*xml, destData);
 }
 
 void MultiEncoderAudioProcessor::setStateInformation (const void* data, int sizeInBytes)
 {
-    ScopedPointer<XmlElement> xmlState (getXmlFromBinary (data, sizeInBytes));
+    std::unique_ptr<XmlElement> xmlState (getXmlFromBinary (data, sizeInBytes));
     if (xmlState != nullptr)
         if (xmlState->hasTagName (parameters.state.getType()))
         {
@@ -351,10 +327,15 @@ void MultiEncoderAudioProcessor::setStateInformation (const void* data, int size
                 else elementColours[i] = Colours::cyan;
             updateColours = true;
 
-            if (parameters.state.hasProperty ("OSCPort"))
+            if (parameters.state.hasProperty ("OSCPort")) // legacy
             {
-                oscReceiver.connect (parameters.state.getProperty ("OSCPort", var (-1)));
+                oscParameterInterface.getOSCReceiver().connect (parameters.state.getProperty ("OSCPort", var (-1)));
+                parameters.state.removeProperty ("OSCPort", nullptr);
             }
+
+            auto oscConfig = parameters.state.getChildWithName ("OSCConfig");
+            if (oscConfig.isValid())
+                oscParameterInterface.setConfig (oscConfig);
         }
 }
 
@@ -405,53 +386,18 @@ void MultiEncoderAudioProcessor::updateQuaternions()
     }
 }
 
-//==============================================================================
-pointer_sized_int MultiEncoderAudioProcessor::handleVstPluginCanDo (int32 index,
-                                                                     pointer_sized_int value, void* ptr, float opt)
-{
-    auto text = (const char*) ptr;
-    auto matches = [=](const char* s) { return strcmp (text, s) == 0; };
-
-    if (matches ("wantsChannelCountNotifications"))
-        return 1;
-    return 0;
-}
 
 //==============================================================================
-void MultiEncoderAudioProcessor::oscMessageReceived (const OSCMessage &message)
-{
-    String prefix ("/" + String(JucePlugin_Name));
-    if (! message.getAddressPattern().toString().startsWith (prefix))
-        return;
-
-    OSCMessage msg (message);
-    msg.setAddressPattern (message.getAddressPattern().toString().substring(String(JucePlugin_Name).length() + 1));
-    oscParams.processOSCMessage (msg);
-}
-
-void MultiEncoderAudioProcessor::oscBundleReceived (const OSCBundle &bundle)
-{
-    for (int i = 0; i < bundle.size(); ++i)
-    {
-        auto elem = bundle[i];
-        if (elem.isMessage())
-            oscMessageReceived (elem.getMessage());
-        else if (elem.isBundle())
-            oscBundleReceived (elem.getBundle());
-    }
-}
-
-//==============================================================================
-AudioProcessorValueTreeState::ParameterLayout MultiEncoderAudioProcessor::createParameterLayout()
+std::vector<std::unique_ptr<RangedAudioParameter>> MultiEncoderAudioProcessor::createParameterLayout()
 {
     // add your audio parameters here
     std::vector<std::unique_ptr<RangedAudioParameter>> params;
 
-    params.push_back (oscParams.createAndAddParameter("inputSetting", "Number of input channels ", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay("inputSetting", "Number of input channels ", "",
                                     NormalisableRange<float> (0.0f, maxNumberOfInputs, 1.0f), startNnumberOfInputs,
                                     [](float value) {return String(value);}, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("orderSetting", "Ambisonics Order", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("orderSetting", "Ambisonics Order", "",
                                      NormalisableRange<float> (0.0f, 8.0f, 1.0f), 0.0f,
                                      [](float value) {
                                          if (value >= 0.5f && value < 1.5f) return "0th";
@@ -465,7 +411,7 @@ AudioProcessorValueTreeState::ParameterLayout MultiEncoderAudioProcessor::create
                                          else return "Auto";},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("useSN3D", "Normalization", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("useSN3D", "Normalization", "",
                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
                                      [](float value)
                                      {
@@ -473,45 +419,103 @@ AudioProcessorValueTreeState::ParameterLayout MultiEncoderAudioProcessor::create
                                          else return "N3D";
                                      }, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter("masterAzimuth", "Master azimuth angle", CharPointer_UTF8 (R"(°)"),
+    params.push_back (OSCParameterInterface::createParameterTheOldWay("masterAzimuth", "Master azimuth angle", CharPointer_UTF8 (R"(°)"),
                                     NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0f,
                                     [](float value) {return String(value, 2);}, nullptr));
-    params.push_back (oscParams.createAndAddParameter("masterElevation", "Master elevation angle", CharPointer_UTF8 (R"(°)"),
-                                    NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0f,
-                                    [](float value) {return String(value, 2);}, nullptr));
-    
-    params.push_back (oscParams.createAndAddParameter("masterRoll", "Master roll angle", CharPointer_UTF8 (R"(°)"),
+    params.push_back (OSCParameterInterface::createParameterTheOldWay("masterElevation", "Master elevation angle", CharPointer_UTF8 (R"(°)"),
                                     NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0f,
                                     [](float value) {return String(value, 2);}, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter("lockedToMaster", "Lock Directions relative to Master", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay("masterRoll", "Master roll angle", CharPointer_UTF8 (R"(°)"),
+                                    NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0f,
+                                    [](float value) {return String(value, 2);}, nullptr));
+
+    params.push_back (OSCParameterInterface::createParameterTheOldWay("lockedToMaster", "Lock Directions relative to Master", "",
                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0f,
                                     [](float value) {return (value >= 0.5f) ? "locked" : "not locked";}, nullptr));
 
     for (int i = 0; i < maxNumberOfInputs; ++i)
     {
-        params.push_back (oscParams.createAndAddParameter("azimuth" + String(i), "Azimuth angle " + String(i + 1), CharPointer_UTF8 (R"(°)"),
+        params.push_back (OSCParameterInterface::createParameterTheOldWay("azimuth" + String(i), "Azimuth angle " + String(i + 1), CharPointer_UTF8 (R"(°)"),
                                         NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
                                         [](float value) {return String(value, 2);}, nullptr));
 
-        params.push_back (oscParams.createAndAddParameter("elevation" + String(i), "Elevation angle " + String(i + 1), CharPointer_UTF8 (R"(°)"),
+        params.push_back (OSCParameterInterface::createParameterTheOldWay("elevation" + String(i), "Elevation angle " + String(i + 1), CharPointer_UTF8 (R"(°)"),
                                         NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
                                         [](float value) {return String(value, 2);}, nullptr));
 
-        params.push_back (oscParams.createAndAddParameter("gain" + String(i), "Gain " + String(i + 1), "dB",
+        params.push_back (OSCParameterInterface::createParameterTheOldWay("gain" + String(i), "Gain " + String(i + 1), "dB",
                                         NormalisableRange<float> (-60.0f, 10.0f, 0.1f), 0.0f,
                                         [](float value) {return (value >= -59.9f) ? String(value, 1) : "-inf";},
                                         nullptr));
 
-        params.push_back (oscParams.createAndAddParameter("mute" + String(i), "Mute input " + String(i + 1), "",
+        params.push_back (OSCParameterInterface::createParameterTheOldWay("mute" + String(i), "Mute input " + String(i + 1), "",
                                         NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0f,
                                         [](float value) {return (value >= 0.5f) ? "muted" : "not muted";}, nullptr));
 
-        params.push_back (oscParams.createAndAddParameter("solo" + String(i), "Solo input " + String(i + 1), "",
+        params.push_back (OSCParameterInterface::createParameterTheOldWay("solo" + String(i), "Solo input " + String(i + 1), "",
                                         NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0f,
                                         [](float value) {return (value >= 0.5f) ? "soloed" : "not soloed";}, nullptr));
     }
 
 
-    return { params.begin(), params.end() };
+    return params;
+}
+
+//==============================================================================
+
+Result MultiEncoderAudioProcessor::loadConfiguration (const File& configFile)
+{
+    ValueTree newSources ("NewSources");
+
+    Result result = ConfigurationHelper::parseFileForLoudspeakerLayout (configFile, newSources, nullptr);
+
+    if (result.wasOk())
+    {
+        int nSrc = 0;
+        const auto nElements = newSources.getNumChildren();
+
+        for (int i = 0; i < nElements; ++i)
+        {
+            auto src = newSources.getChild (i);
+            const int ch = src.getProperty ("Channel");
+            const bool isImaginary = src.getProperty ("Imaginary");
+            if (! isImaginary && ch > nSrc)
+                nSrc = ch;
+        }
+
+        DBG (nSrc << " Sources!");
+        parameters.getParameterAsValue ("inputSetting").setValue (nSrc);
+
+        for (int s = 0; s < nSrc; ++s)
+            parameters.getParameterAsValue ("mute" + String (s)).setValue (1);
+
+        for (int e = 0; e < nElements; ++e)
+        {
+            const auto src = newSources.getChild (e);
+            const int ch = static_cast<int> (src.getProperty ("Channel", 0)) - 1;
+            const bool isImaginary = src.getProperty ("Imaginary");
+
+            if (isImaginary || ch < 0 || ch >= 64)
+                continue;
+
+
+            parameters.getParameterAsValue ("mute" + String (ch)).setValue (0);
+
+            auto azi = src.getProperty ("Azimuth", 0.0f);
+            parameters.getParameterAsValue ("azimuth" + String (ch)).setValue (azi);
+            auto ele = src.getProperty ("Elevation", 0.0f);
+            parameters.getParameterAsValue ("elevation" + String (ch)).setValue (ele);
+        }
+    }
+
+    return result;
+}
+
+
+void MultiEncoderAudioProcessor::setLastDir (File newLastDir)
+{
+    lastDir = newLastDir;
+    const var v (lastDir.getFullPathName());
+    properties->setValue ("presetFolder", v);
 }
