@@ -22,32 +22,29 @@
 
 #pragma once
 
-#define maxNumberOfInputs 64
-#define startNnumberOfInputs 5
-
-
 #include "../JuceLibraryCode/JuceHeader.h"
 #include "../../resources/Quaternion.h"
 #include "../../resources/efficientSHvanilla.h"
 #include "../../resources/ambisonicTools.h"
-#include "../../resources/IOHelper.h"
+#include "../../resources/AudioProcessorBase.h"
 #include "../../resources/Conversions.h"
 
-// ===== OSC ====
-#include "../../resources/OSCParameterInterface.h"
-#include "../../resources/OSCReceiverPlus.h"
+#define CONFIGURATIONHELPER_ENABLE_LOUDSPEAKERLAYOUT_METHODS 1
+#include "../../resources/ConfigurationHelper.h"
 
+#define ProcessorClass MultiEncoderAudioProcessor
+
+constexpr int maxNumberOfInputs = 64;
+constexpr int startNnumberOfInputs = 5;
 
 //==============================================================================
 /**
 */
-class MultiEncoderAudioProcessor  : public AudioProcessor,
-                                    public AudioProcessorValueTreeState::Listener,
-                                    public IOHelper<IOTypes::AudioChannels<maxNumberOfInputs>, IOTypes::Ambisonics<>>,
-                                    public VSTCallbackHandler,
-                                    private OSCReceiver::Listener<OSCReceiver::RealtimeCallback>
+class MultiEncoderAudioProcessor  : public AudioProcessorBase<IOTypes::AudioChannels<maxNumberOfInputs>, IOTypes::Ambisonics<>>
 {
 public:
+    constexpr static int numberOfInputChannels = 64;
+    constexpr static int numberOfOutputChannels = 64;
     //==============================================================================
     MultiEncoderAudioProcessor();
     ~MultiEncoderAudioProcessor();
@@ -56,23 +53,12 @@ public:
     void prepareToPlay (double sampleRate, int samplesPerBlock) override;
     void releaseResources() override;
 
-   #ifndef JucePlugin_PreferredChannelConfigurations
-    bool isBusesLayoutSupported (const BusesLayout& layouts) const override;
-   #endif
-
     void processBlock (AudioSampleBuffer&, MidiBuffer&) override;
 
     //==============================================================================
     AudioProcessorEditor* createEditor() override;
     bool hasEditor() const override;
 
-    //==============================================================================
-    const String getName() const override;
-
-    bool acceptsMidi() const override;
-    bool producesMidi() const override;
-    double getTailLengthSeconds() const override;
-
     //==============================================================================
     int getNumPrograms() override;
     int getCurrentProgram() override;
@@ -86,22 +72,14 @@ public:
 
     void parameterChanged (const String &parameterID, float newValue) override;
 
-    //======== PluginCanDo =========================================================
-    pointer_sized_int handleVstManufacturerSpecific (int32 index, pointer_sized_int value,
-                                                     void* ptr, float opt) override { return 0; };
-    pointer_sized_int handleVstPluginCanDo (int32 index, pointer_sized_int value,
-                                            void* ptr, float opt) override;
-    //==============================================================================
-
-    //======== OSC =================================================================
-    void oscMessageReceived (const OSCMessage &message) override;
-    void oscBundleReceived (const OSCBundle &bundle) override;
-    OSCReceiverPlus& getOSCReceiver () { return oscReceiver; }
-    //==============================================================================
 
     //======= Parameters ===========================================================
-    AudioProcessorValueTreeState::ParameterLayout createParameterLayout();
+    std::vector<std::unique_ptr<RangedAudioParameter>> createParameterLayout();
+
     //==============================================================================
+    Result loadConfiguration (const File& configFile);
+    void setLastDir (File newLastDir);
+    File getLastDir() { return lastDir; };
 
     float xyzGrab[3];
     float xyz[maxNumberOfInputs][3];
@@ -139,9 +117,8 @@ public:
 
 private:
     //==============================================================================
-    OSCParameterInterface oscParams;
-    OSCReceiverPlus oscReceiver;
-    AudioProcessorValueTreeState parameters;
+    File lastDir;
+    std::unique_ptr<PropertiesFile> properties;
 
     bool processorUpdatingParams;
 
