@@ -25,23 +25,19 @@
 #include "../JuceLibraryCode/JuceHeader.h"
 #include "../../resources/efficientSHvanilla.h"
 #include "../../resources/ambisonicTools.h"
-#include "../../resources/IOHelper.h"
+#include "../../resources/AudioProcessorBase.h"
 #include "../../resources/Conversions.h"
 
-// ===== OSC ====
-#include "../../resources/OSCParameterInterface.h"
-#include "../../resources/OSCReceiverPlus.h"
+#define ProcessorClass ProbeDecoderAudioProcessor
 
 //==============================================================================
 /**
 */
-class ProbeDecoderAudioProcessor  : public AudioProcessor,
-                                    public AudioProcessorValueTreeState::Listener,
-                                    public IOHelper<IOTypes::Ambisonics<>, IOTypes::AudioChannels<1>>,
-                                    public VSTCallbackHandler,
-                                    private OSCReceiver::Listener<OSCReceiver::RealtimeCallback>
+class ProbeDecoderAudioProcessor  : public AudioProcessorBase<IOTypes::Ambisonics<>, IOTypes::AudioChannels<1>>
 {
 public:
+    constexpr static int numberOfInputChannels = 64;
+    constexpr static int numberOfOutputChannels = 1;
     //==============================================================================
     ProbeDecoderAudioProcessor();
     ~ProbeDecoderAudioProcessor();
@@ -50,23 +46,12 @@ public:
     void prepareToPlay (double sampleRate, int samplesPerBlock) override;
     void releaseResources() override;
 
-   #ifndef JucePlugin_PreferredChannelConfigurations
-    bool isBusesLayoutSupported (const BusesLayout& layouts) const override;
-   #endif
-
     void processBlock (AudioSampleBuffer&, MidiBuffer&) override;
 
     //==============================================================================
     AudioProcessorEditor* createEditor() override;
     bool hasEditor() const override;
 
-    //==============================================================================
-    const String getName() const override;
-
-    bool acceptsMidi() const override;
-    bool producesMidi() const override;
-    double getTailLengthSeconds() const override;
-
     //==============================================================================
     int getNumPrograms() override;
     int getCurrentProgram() override;
@@ -80,23 +65,10 @@ public:
 
     void parameterChanged (const String &parameterID, float newValue) override;
 
-    //======== PluginCanDo =========================================================
-    pointer_sized_int handleVstManufacturerSpecific (int32 index, pointer_sized_int value,
-                                                     void* ptr, float opt) override { return 0; };
-    pointer_sized_int handleVstPluginCanDo (int32 index, pointer_sized_int value,
-                                            void* ptr, float opt) override;
-    //==============================================================================
-
-    //======== OSC =================================================================
-    void oscMessageReceived (const OSCMessage &message) override;
-    void oscBundleReceived (const OSCBundle &bundle) override;
-    OSCReceiverPlus& getOSCReceiver () { return oscReceiver; }
-    //==============================================================================
-
     //======= Parameters ===========================================================
-    AudioProcessorValueTreeState::ParameterLayout createParameterLayout();
-    //==============================================================================
+    std::vector<std::unique_ptr<RangedAudioParameter>> createParameterLayout();
 
+    //==============================================================================
     float *orderSetting;
     float *useSN3D;
 
@@ -104,10 +76,7 @@ public:
 
 private:
     //==============================================================================
-    OSCParameterInterface oscParams;
-    OSCReceiverPlus oscReceiver;
-    AudioProcessorValueTreeState parameters;
-    
+
     float *azimuth;
     float *elevation;
 
