@@ -25,7 +25,7 @@
 
 //==============================================================================
 RoomEncoderAudioProcessorEditor::RoomEncoderAudioProcessorEditor (RoomEncoderAudioProcessor& p, AudioProcessorValueTreeState& vts)
-: AudioProcessorEditor (&p), footer (p.getOSCReceiver()), processor (p), valueTreeState(vts),
+: AudioProcessorEditor (&p), footer (p.getOSCParameterInterface()), processor (p), valueTreeState (vts),
 sourceElement(*valueTreeState.getParameter("sourceX"), valueTreeState.getParameterRange("sourceX"),
               *valueTreeState.getParameter("sourceY"), valueTreeState.getParameterRange("sourceY"),
               *valueTreeState.getParameter("sourceZ"), valueTreeState.getParameterRange("sourceZ")),
@@ -34,19 +34,21 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
               *valueTreeState.getParameter("listenerZ"), valueTreeState.getParameterRange("listenerZ"))
 
 {
-
-    setSize (800, 600);
+    setSize (820, 600);
     setLookAndFeel (&globalLaF);
+
+    toolTipWin.setLookAndFeel (&globalLaF);
     toolTipWin.setMillisecondsBeforeTipAppears(500);
     toolTipWin.setOpaque (false);
 
-    addAndMakeVisible(&title);
-    title.setTitle(String("Room"),String("Encoder"));
-    title.setFont(globalLaF.robotoBold,globalLaF.robotoLight);
-    cbNormalizationAttachement = new ComboBoxAttachment(valueTreeState,"useSN3D", *title.getOutputWidgetPtr()->getNormCbPointer());
-    cbOrderAttachement = new ComboBoxAttachment(valueTreeState,"orderSetting", *title.getOutputWidgetPtr()->getOrderCbPointer());
-    cbDirectivityOrderAttachment = new ComboBoxAttachment(valueTreeState,"directivityOrderSetting", *title.getInputWidgetPtr()->getOrderCbPointer());
-    cbDirectivityNormalizationAttachment = new ComboBoxAttachment(valueTreeState, "inputIsSN3D", *title.getInputWidgetPtr()->getNormCbPointer());
+    addAndMakeVisible (&title);
+    title.setTitle (String ("Room"), String ("Encoder"));
+    title.setFont (globalLaF.robotoBold, globalLaF.robotoLight);
+
+    cbNormalizationAttachement.reset (new ComboBoxAttachment (valueTreeState, "useSN3D", *title.getOutputWidgetPtr()->getNormCbPointer()));
+    cbOrderAttachement.reset (new ComboBoxAttachment (valueTreeState, "orderSetting", *title.getOutputWidgetPtr()->getOrderCbPointer()));
+    cbDirectivityOrderAttachment.reset (new ComboBoxAttachment (valueTreeState, "directivityOrderSetting", *title.getInputWidgetPtr()->getOrderCbPointer()));
+    cbDirectivityNormalizationAttachment.reset (new ComboBoxAttachment (valueTreeState, "inputIsSN3D", *title.getInputWidgetPtr()->getNormCbPointer()));
     addAndMakeVisible (&footer);
 
 
@@ -103,7 +105,7 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
     lbSourceZ.setText("z");
 
     addAndMakeVisible(&slRoomX);
-    slRoomXAttachment = new SliderAttachment(valueTreeState,"roomX", slRoomX);
+    slRoomXAttachment.reset (new SliderAttachment(valueTreeState,"roomX", slRoomX));
     slRoomX.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slRoomX.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
     slRoomX.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[0]);
@@ -112,7 +114,7 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
     slRoomX.addListener(this);
 
     addAndMakeVisible(&slRoomY);
-    slRoomYAttachment = new SliderAttachment(valueTreeState, "roomY", slRoomY);
+    slRoomYAttachment.reset (new SliderAttachment(valueTreeState, "roomY", slRoomY));
     slRoomY.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slRoomY.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
     slRoomY.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[0]);
@@ -121,7 +123,7 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
     slRoomY.addListener(this);
 
     addAndMakeVisible(&slRoomZ);
-    slRoomZAttachment = new SliderAttachment(valueTreeState, "roomZ", slRoomZ);
+    slRoomZAttachment.reset (new SliderAttachment(valueTreeState, "roomZ", slRoomZ));
     slRoomZ.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slRoomZ.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
     slRoomZ.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[0]);
@@ -141,7 +143,7 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
     zyPlane.useAutoScale(false);
 
     addAndMakeVisible(&slSourceX);
-    slSourceXAttachment = new SliderAttachment(valueTreeState, "sourceX", slSourceX);
+    slSourceXAttachment.reset (new SliderAttachment(valueTreeState, "sourceX", slSourceX));
     slSourceX.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slSourceX.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
     slSourceX.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[2]);
@@ -150,7 +152,7 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
     slSourceX.addListener(this);
 
     addAndMakeVisible(&slSourceY);
-    slSourceYAttachment = new SliderAttachment(valueTreeState, "sourceY", slSourceY);
+    slSourceYAttachment.reset (new SliderAttachment(valueTreeState, "sourceY", slSourceY));
     slSourceY.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slSourceY.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
     slSourceY.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[2]);
@@ -159,7 +161,7 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
     slSourceY.addListener(this);
 
     addAndMakeVisible(&slSourceZ);
-    slSourceZAttachment = new SliderAttachment(valueTreeState, "sourceZ", slSourceZ);
+    slSourceZAttachment.reset (new SliderAttachment(valueTreeState, "sourceZ", slSourceZ));
     slSourceZ.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slSourceZ.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
     slSourceZ.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[2]);
@@ -168,7 +170,7 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
     slSourceZ.addListener(this);
 
     addAndMakeVisible(&slListenerX);
-    slListenerXAttachment = new SliderAttachment(valueTreeState, "listenerX", slListenerX);
+    slListenerXAttachment.reset (new SliderAttachment(valueTreeState, "listenerX", slListenerX));
     slListenerX.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slListenerX.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
     slListenerX.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[1]);
@@ -178,7 +180,7 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
     slListenerX.addListener(this);
 
     addAndMakeVisible(&slListenerY);
-    slListenerYAttachment = new SliderAttachment(valueTreeState,"listenerY", slListenerY);
+    slListenerYAttachment.reset (new SliderAttachment(valueTreeState,"listenerY", slListenerY));
     slListenerY.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slListenerY.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
     slListenerY.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[1]);
@@ -187,7 +189,7 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
     slListenerY.addListener(this);
 
     addAndMakeVisible(&slListenerZ);
-    slListenerZAttachment = new SliderAttachment(valueTreeState,"listenerZ", slListenerZ);
+    slListenerZAttachment.reset (new SliderAttachment (valueTreeState,"listenerZ", slListenerZ));
     slListenerZ.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slListenerZ.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
     slListenerZ.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[1]);
@@ -201,17 +203,68 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
     addAndMakeVisible(&lbNumReflections);
     lbNumReflections.setText("Number of Reflections");
     addAndMakeVisible(&slNumReflections);
-    slNumReflectionsAttachment = new SliderAttachment(valueTreeState,"numRefl", slNumReflections);
+    slNumReflectionsAttachment.reset (new SliderAttachment(valueTreeState,"numRefl", slNumReflections));
     slNumReflections.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slNumReflections.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
-    slNumReflections.setColour (Slider::rotarySliderOutlineColourId, Colours::lightgrey);
+    slNumReflections.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[1]);
 
 
+    addAndMakeVisible (lbWallAttenuation);
+    lbWallAttenuation.setText ("Additional Attenuation", true, Justification::left);
+
+    addAndMakeVisible (slWallAttenuationFront);
+    slWallAttenuationFrontAttachment.reset (new SliderAttachment (valueTreeState, "wallAttenuationFront", slWallAttenuationFront));
+    slWallAttenuationFront.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
+    slWallAttenuationFront.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
+    slWallAttenuationFront.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[2]);
+    addAndMakeVisible (lbWallAttenuationFront);
+    lbWallAttenuationFront.setText ("Front");
+
+    addAndMakeVisible (slWallAttenuationBack);
+    slWallAttenuationBackAttachment.reset (new SliderAttachment (valueTreeState, "wallAttenuationBack", slWallAttenuationBack));
+    slWallAttenuationBack.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
+    slWallAttenuationBack.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
+    slWallAttenuationBack.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[2]);
+    addAndMakeVisible (lbWallAttenuationBack);
+    lbWallAttenuationBack.setText ("Back");
+
+    addAndMakeVisible (slWallAttenuationLeft);
+    slWallAttenuationLeftAttachment.reset (new SliderAttachment (valueTreeState, "wallAttenuationLeft", slWallAttenuationLeft));
+    slWallAttenuationLeft.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
+    slWallAttenuationLeft.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
+    slWallAttenuationLeft.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[2]);
+    addAndMakeVisible (lbWallAttenuationLeft);
+    lbWallAttenuationLeft.setText ("Left");
+
+    addAndMakeVisible (slWallAttenuationRight);
+    slWallAttenuationRightAttachment.reset (new SliderAttachment (valueTreeState, "wallAttenuationRight", slWallAttenuationRight));
+    slWallAttenuationRight.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
+    slWallAttenuationRight.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
+    slWallAttenuationRight.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[2]);
+    addAndMakeVisible (lbWallAttenuationRight);
+    lbWallAttenuationRight.setText ("Right");
+
+    addAndMakeVisible (slWallAttenuationCeiling);
+    slWallAttenuationCeilingAttachment.reset (new SliderAttachment (valueTreeState, "wallAttenuationCeiling", slWallAttenuationCeiling));
+    slWallAttenuationCeiling.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
+    slWallAttenuationCeiling.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
+    slWallAttenuationCeiling.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[2]);
+    addAndMakeVisible (lbWallAttenuationCeiling);
+    lbWallAttenuationCeiling.setText ("Ceiling");
+
+    addAndMakeVisible (slWallAttenuationFloor);
+    slWallAttenuationFloorAttachment.reset (new SliderAttachment (valueTreeState, "wallAttenuationFloor", slWallAttenuationFloor));
+    slWallAttenuationFloor.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
+    slWallAttenuationFloor.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
+    slWallAttenuationFloor.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[2]);
+    addAndMakeVisible (lbWallAttenuationFloor);
+    lbWallAttenuationFloor.setText ("Floor");
+
     addAndMakeVisible(&lbReflCoeff);
-    lbReflCoeff.setText("Reflection Attenuation");
+    lbReflCoeff.setText("Overall Attenuation");
 
     addAndMakeVisible(&slReflCoeff);
-    slReflCoeffAttachment = new SliderAttachment(valueTreeState,"reflCoeff", slReflCoeff);
+    slReflCoeffAttachment.reset (new SliderAttachment(valueTreeState,"reflCoeff", slReflCoeff));
     slReflCoeff.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slReflCoeff.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
     slReflCoeff.setColour (Slider::rotarySliderOutlineColourId, Colours::lightgrey);
@@ -227,7 +280,7 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
     lbHSG.setText("Gain");
 
     addAndMakeVisible(&slLowShelfFreq);
-    slLowShelfFreqAttachment = new SliderAttachment(valueTreeState,"lowShelfFreq", slLowShelfFreq);
+    slLowShelfFreqAttachment.reset (new SliderAttachment(valueTreeState,"lowShelfFreq", slLowShelfFreq));
     slLowShelfFreq.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slLowShelfFreq.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
     slLowShelfFreq.setColour (Slider::rotarySliderOutlineColourId, Colours::cyan);
@@ -235,7 +288,7 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
 
 
     addAndMakeVisible(&slLowShelfGain);
-    slLowShelfGainAttachment = new SliderAttachment(valueTreeState,"lowShelfGain", slLowShelfGain);
+    slLowShelfGainAttachment.reset (new SliderAttachment(valueTreeState,"lowShelfGain", slLowShelfGain));
     slLowShelfGain.setTextValueSuffix (" dB");
     slLowShelfGain.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slLowShelfGain.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
@@ -243,14 +296,14 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
 
 
     addAndMakeVisible(&slHighShelfFreq);
-    slHighShelfFreqAttachment = new SliderAttachment(valueTreeState,"highShelfFreq", slHighShelfFreq);
+    slHighShelfFreqAttachment.reset (new SliderAttachment(valueTreeState,"highShelfFreq", slHighShelfFreq));
     slHighShelfFreq.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slHighShelfFreq.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
     slHighShelfFreq.setColour (Slider::rotarySliderOutlineColourId, Colours::orangered);
     slHighShelfFreq.setTooltip("high shelf freq");
 
     addAndMakeVisible(&slHighShelfGain);
-    slHighShelfGainAttachment = new SliderAttachment(valueTreeState,"highShelfGain", slHighShelfGain);
+    slHighShelfGainAttachment.reset (new SliderAttachment(valueTreeState,"highShelfGain", slHighShelfGain));
     slHighShelfGain.setTextValueSuffix (" dB");
     slHighShelfGain.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slHighShelfGain.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
@@ -275,26 +328,43 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
     cbSyncChannel.addItem("2", 3);
     cbSyncChannel.addItem("3", 4);
     cbSyncChannel.addItem("4", 5);
-    cbSyncChannelAttachment = new ComboBoxAttachment(valueTreeState, "syncChannel", cbSyncChannel);
+    cbSyncChannelAttachment.reset (new ComboBoxAttachment(valueTreeState, "syncChannel", cbSyncChannel));
 
     addAndMakeVisible(&lbSyncChannel);
     lbSyncChannel.setText("Synchronize to Channel");
 
     addAndMakeVisible(&tbSyncRoomSize);
-    tbSyncRoomSizeAttachment = new ButtonAttachment(valueTreeState, "syncRoomSize", tbSyncRoomSize);
+    tbSyncRoomSizeAttachment.reset (new ButtonAttachment(valueTreeState, "syncRoomSize", tbSyncRoomSize));
     tbSyncRoomSize.setButtonText("Room Dimensions");
     tbSyncRoomSize.setColour (ToggleButton::tickColourId, globalLaF.ClWidgetColours[0]);
 
     addAndMakeVisible(&tbSyncListener);
-    tbSyncListenerAttachment = new ButtonAttachment(valueTreeState, "syncListener", tbSyncListener);
+    tbSyncListenerAttachment.reset (new ButtonAttachment(valueTreeState, "syncListener", tbSyncListener));
     tbSyncListener.setButtonText("Listener Position");
     tbSyncListener.setColour (ToggleButton::tickColourId, globalLaF.ClWidgetColours[1]);
 
     addAndMakeVisible(&tbSyncReflection);
-    tbSyncReflectionAttachment = new ButtonAttachment(valueTreeState, "syncReflection", tbSyncReflection);
+    tbSyncReflectionAttachment.reset (new ButtonAttachment(valueTreeState, "syncReflection", tbSyncReflection));
     tbSyncReflection.setButtonText("Reflection Properties");
     tbSyncReflection.setColour (ToggleButton::tickColourId, Colours::orangered);
 
+    addAndMakeVisible (&tbDirectPathUnityGain);
+    tbDirectPathUnityGainAttachment.reset (new ButtonAttachment (valueTreeState, "directPathUnityGain", tbDirectPathUnityGain));
+    tbDirectPathUnityGain.setButtonText ("Direct Path Unity Gain");
+    tbDirectPathUnityGain.setColour (ToggleButton::tickColourId, globalLaF.ClWidgetColours[0]);
+
+    addAndMakeVisible (&tbDirectPathZeroDelay);
+    tbDirectPathZeroDelayAttachment.reset (new ButtonAttachment (valueTreeState, "directPathZeroDelay", tbDirectPathZeroDelay));
+    tbDirectPathZeroDelay.setButtonText ("Direct Path Zero Delay");
+    tbDirectPathZeroDelay.setColour (ToggleButton::tickColourId, globalLaF.ClWidgetColours[1]);
+    rv.setZeroDelay (tbDirectPathZeroDelay.getToggleState());
+    tbDirectPathZeroDelay.addListener (this);
+
+    addAndMakeVisible (&tbRenderDirectPath);
+    tbRenderDirectPathAttachment.reset (new ButtonAttachment (valueTreeState, "renderDirectPath", tbRenderDirectPath));
+    tbRenderDirectPath.setButtonText ("Render Direct Path");
+    tbRenderDirectPath.setColour (ToggleButton::tickColourId, globalLaF.ClWidgetColours[2]);
+    tbRenderDirectPath.addListener (this);
 
     startTimer(20);
 }
@@ -335,7 +405,7 @@ void RoomEncoderAudioProcessorEditor::resized()
 
     area.removeFromLeft(leftRightMargin);
     area.removeFromRight(leftRightMargin);
-    Rectangle<int> headerArea = area.removeFromTop    (headerHeight);
+    Rectangle<int> headerArea = area.removeFromTop (headerHeight);
     title.setBounds (headerArea);
     area.removeFromTop(10);
 
@@ -358,7 +428,7 @@ void RoomEncoderAudioProcessorEditor::resized()
 
 
 
-    Rectangle<int> propArea (area.removeFromRight(9*rotSliderWidth+8*rotSliderSpacing));
+    Rectangle<int> propArea (area.removeFromRight (9 * rotSliderWidth + 8 * rotSliderSpacing + 20));
     {
         Rectangle<int> coordinateArea (propArea.removeFromTop(100));
 
@@ -379,7 +449,7 @@ void RoomEncoderAudioProcessorEditor::resized()
         roomArea.removeFromLeft(rotSliderSpacing);
         lbRoomZ.setBounds (roomArea.removeFromLeft(rotSliderWidth));
 
-        coordinateArea.removeFromLeft(rotSliderSpacing);
+        coordinateArea.removeFromLeft (20);
         Rectangle<int> sourceArea (coordinateArea.removeFromLeft(3*rotSliderWidth+2*rotSliderSpacing));
         gcSourcePosition.setBounds (sourceArea);
         sourceArea.removeFromTop(25);
@@ -397,7 +467,7 @@ void RoomEncoderAudioProcessorEditor::resized()
         sourceArea.removeFromLeft(rotSliderSpacing);
         lbSourceZ.setBounds (sourceArea.removeFromLeft(rotSliderWidth));
 
-        coordinateArea.removeFromLeft(rotSliderSpacing);
+        coordinateArea.removeFromLeft (20);
         Rectangle<int> listenerArea (coordinateArea.removeFromLeft(3*rotSliderWidth+2*rotSliderSpacing));
         gcListenerPosition.setBounds (listenerArea);
         listenerArea.removeFromTop(25);
@@ -416,13 +486,22 @@ void RoomEncoderAudioProcessorEditor::resized()
         lbListenerZ.setBounds (listenerArea.removeFromLeft(rotSliderWidth));
     }
 
-    propArea.removeFromTop(20);
+    propArea.removeFromTop (10);
     gcReflectionProperties.setBounds(propArea);
-    propArea.removeFromTop(20);
+    propArea.removeFromTop (20);
+
+    auto buttonRow = propArea.removeFromBottom (20);
+    tbDirectPathZeroDelay.setBounds (buttonRow.removeFromLeft (120));
+    buttonRow.removeFromLeft (10);
+    tbDirectPathUnityGain.setBounds (buttonRow.removeFromLeft (120));
+    buttonRow.removeFromLeft (10);
+    tbRenderDirectPath.setBounds (buttonRow.removeFromLeft (120));
+
+
     Rectangle<int> fvCol (propArea.removeFromLeft(330));
 
     { // 120
-        Rectangle<int> fvRow (fvCol.removeFromTop(120));
+        Rectangle<int> fvRow (fvCol.removeFromTop(130));
         fv.setBounds(fvRow);
 
         fvCol.removeFromTop(10);
@@ -438,6 +517,8 @@ void RoomEncoderAudioProcessorEditor::resized()
         sliderRow.removeFromRight(sliderSpacing);
         slHighShelfFreq.setBounds(sliderRow.removeFromRight(rotSliderWidth));
 
+        slReflCoeff.setBounds (sliderRow);
+
         sliderRow = fvCol.removeFromTop(labelHeight);
         sliderRow.removeFromLeft(20);
         lbLSF.setBounds(sliderRow.removeFromLeft(rotSliderWidth));
@@ -448,14 +529,44 @@ void RoomEncoderAudioProcessorEditor::resized()
         lbHSG.setBounds(sliderRow.removeFromRight(rotSliderWidth));
         sliderRow.removeFromRight(sliderSpacing);
         lbHSF.setBounds(sliderRow.removeFromRight(rotSliderWidth));
+
+        lbReflCoeff.setBounds (sliderRow);
     }
 
-    propArea.removeFromTop(20);
-    slReflCoeff.setBounds(propArea.removeFromTop(rotSliderHeight+10));
-    lbReflCoeff.setBounds(propArea.removeFromTop(labelHeight));
-    propArea.removeFromTop(20);
-    slNumReflections.setBounds(propArea.removeFromTop(rotSliderHeight+10));
-    lbNumReflections.setBounds(propArea.removeFromTop(labelHeight));
+    auto wallAttenArea = propArea.removeFromTop (135);
+    lbWallAttenuation.setBounds (wallAttenArea.removeFromTop (20));
+    auto sliderRow = wallAttenArea.removeFromTop (rotSliderHeight - 10);
+    slWallAttenuationFront.setBounds (sliderRow.removeFromLeft (rotSliderWidth));
+    sliderRow.removeFromLeft (5);
+    slWallAttenuationLeft.setBounds (sliderRow.removeFromLeft (rotSliderWidth));
+    sliderRow.removeFromLeft (5);
+    slWallAttenuationRight.setBounds (sliderRow.removeFromLeft (rotSliderWidth));
+
+    sliderRow = wallAttenArea.removeFromTop (12);
+    lbWallAttenuationFront.setBounds (sliderRow.removeFromLeft (rotSliderWidth));
+    sliderRow.removeFromLeft (5);
+    lbWallAttenuationLeft.setBounds (sliderRow.removeFromLeft (rotSliderWidth));
+    sliderRow.removeFromLeft (5);
+    lbWallAttenuationRight.setBounds (sliderRow.removeFromLeft (rotSliderWidth));
+
+    sliderRow = wallAttenArea.removeFromTop (rotSliderHeight - 10);
+    slWallAttenuationBack.setBounds (sliderRow.removeFromLeft (rotSliderWidth));
+    sliderRow.removeFromLeft (5);
+    slWallAttenuationCeiling.setBounds (sliderRow.removeFromLeft (rotSliderWidth));
+    sliderRow.removeFromLeft (5);
+    slWallAttenuationFloor.setBounds (sliderRow.removeFromLeft (rotSliderWidth));
+
+    sliderRow = wallAttenArea.removeFromTop (12);
+    lbWallAttenuationBack.setBounds (sliderRow.removeFromLeft (rotSliderWidth));
+    sliderRow.removeFromLeft (5);
+    lbWallAttenuationCeiling.setBounds (sliderRow.removeFromLeft (rotSliderWidth));
+    sliderRow.removeFromLeft (5);
+    lbWallAttenuationFloor.setBounds (sliderRow.removeFromLeft (rotSliderWidth));
+
+
+    propArea.removeFromTop (5);
+    slNumReflections.setBounds (propArea.removeFromTop (rotSliderHeight));
+    lbNumReflections.setBounds (propArea.removeFromTop (labelHeight));
 
 
     area.removeFromRight(10);
@@ -480,19 +591,23 @@ void RoomEncoderAudioProcessorEditor::sliderValueChanged(Slider *slider)
     }
 }
 
+void RoomEncoderAudioProcessorEditor::buttonStateChanged (Button *button)
+{
+    if (button == &tbDirectPathZeroDelay)
+        rv.setZeroDelay (tbDirectPathZeroDelay.getToggleState());
+}
+
 void RoomEncoderAudioProcessorEditor::timerCallback()
 {
     // === update titleBar widgets according to available input/output channel counts
-    int maxInSize, maxOutSize;
-    processor.getMaxSize(maxInSize, maxOutSize);
-    title.setMaxSize(maxInSize, maxOutSize);
+    title.setMaxSize (processor.getMaxSize());
     // ==========================================
 
     if (processor.updateFv)
     {
-        fv.setOverallGainInDecibels(*valueTreeState.getRawParameterValue("reflCoeff"));
+        fv.setOverallGainInDecibels (*valueTreeState.getRawParameterValue("reflCoeff"));
+        fv.setSampleRate (processor.getSampleRate());
         processor.updateFv = false;
-        fv.repaint();
     }
 
     if (processor.repaintPositionPlanes.get())
