@@ -25,17 +25,13 @@
 #pragma once
 
 #include "../JuceLibraryCode/JuceHeader.h"
-#include "../../resources/Quaternion.h"
 #include "../../resources/interpLagrangeWeights.h"
 #include "../../resources/efficientSHvanilla.h"
 #include "reflections.h"
 #include "../../resources/ambisonicTools.h"
-#include "../../resources/IOHelper.h"
+#include "../../resources/AudioProcessorBase.h"
 #include "../../resources/customComponents/FilterVisualizer.h"
 
-// ===== OSC ====
-#include "../../resources/OSCParameterInterface.h"
-#include "../../resources/OSCReceiverPlus.h"
 
 #ifdef JUCE_MAC
 #define VIMAGE_H // avoid namespace clashes
@@ -50,16 +46,20 @@
 # define IIRfloat_elements() 1
 #endif /* JUCE_USE_SIMD */
 
+#define ProcessorClass RoomEncoderAudioProcessor
+
 const int mSig[] = {1,-1};
 using namespace juce::dsp;
 
-struct RoomParams {
+struct RoomParams
+{
     bool validRoomData = false;
     bool validListenerData = false;
     bool validReflectionData = false;
     float roomX, roomY, roomZ;
     float listenerX, listenerY, listenerZ;
     float reflCoeff, lowShelfFreq, lowShelfGain, highShelfFreq, highShelfGain, numRefl;
+    float wallAttenuationFront, wallAttenuationBack, wallAttenuationLeft, wallAttenuationRight, wallAttenuationCeiling, wallAttenuationFloor;
 };
 
 struct SharedParams {
@@ -73,42 +73,47 @@ struct SharedParams {
     Array<RoomParams> rooms;
 };
 
+struct ReflectionProperty
+{
+    const int x; // coordinate in image source space
+    const int y; // coordinate in image source space
+    const int z; // coordinate in image source space
+    const int order; // image source order
+    const int xPlusReflections; // number of reflections at front wall
+    const int xMinusReflections; // number of reflections at back wall
+    const int yPlusReflections; // number of reflections at left wall
+    const int yMinusReflections; // number of reflections at right wall
+    const int zPlusReflections; // number of reflections at ceiling
+    const int zMinusReflections; // number of reflections at floor
+};
+
 //==============================================================================
 /**
 */
-class RoomEncoderAudioProcessor  :  public AudioProcessor,
-                                    public AudioProcessorValueTreeState::Listener,
-                                    private Timer,
-                                    public IOHelper<IOTypes::Ambisonics<>, IOTypes::Ambisonics<>>,
-                                    public VSTCallbackHandler,
-                                    private OSCReceiver::Listener<OSCReceiver::RealtimeCallback>
+class RoomEncoderAudioProcessor  :  public AudioProcessorBase<IOTypes::Ambisonics<>, IOTypes::Ambisonics<>>,
+                                    private Timer
 {
 public:
+    constexpr static int numberOfInputChannels = 64;
+    constexpr static int numberOfOutputChannels = 64;
     //==============================================================================
     RoomEncoderAudioProcessor();
     ~RoomEncoderAudioProcessor();
 
+    //==============================================================================
+
+    void initializeReflectionList();
+
     //==============================================================================
     void prepareToPlay (double sampleRate, int samplesPerBlock) override;
     void releaseResources() override;
 
-   #ifndef JucePlugin_PreferredChannelConfigurations
-    bool isBusesLayoutSupported (const BusesLayout& layouts) const override;
-   #endif
-
     void processBlock (AudioSampleBuffer&, MidiBuffer&) override;
 
     //==============================================================================
     AudioProcessorEditor* createEditor() override;
     bool hasEditor() const override;
 
-    //==============================================================================
-    const String getName() const override;
-
-    bool acceptsMidi() const override;
-    bool producesMidi() const override;
-    double getTailLengthSeconds() const override;
-
     //==============================================================================
     int getNumPrograms() override;
     int getCurrentProgram() override;
@@ -122,28 +127,13 @@ public:
 
     void parameterChanged (const String &parameterID, float newValue) override;
 
-    //======== PluginCanDo =========================================================
-    pointer_sized_int handleVstManufacturerSpecific (int32 index, pointer_sized_int value,
-                                                     void* ptr, float opt) override { return 0; };
-    pointer_sized_int handleVstPluginCanDo (int32 index, pointer_sized_int value,
-                                            void* ptr, float opt) override;
-    //==============================================================================
-
-    //======== OSC =================================================================
-    void oscMessageReceived (const OSCMessage &message) override;
-    void oscBundleReceived (const OSCBundle &bundle) override;
-    OSCReceiverPlus& getOSCReceiver () { return oscReceiver; }
-    //==============================================================================
 
     //======= Parameters ===========================================================
-    AudioProcessorValueTreeState::ParameterLayout createParameterLayout();
+    std::vector<std::unique_ptr<RangedAudioParameter>> createParameterLayout();
+
     //==============================================================================
-
     double oldDelay[nImgSrc];
-    //float oldRGain[nImgSrc];
     float allGains[nImgSrc];
-    //float* oldDelayPtr;
-
 
     //filter coefficients
     IIR::Coefficients<float>::Ptr lowShelfCoefficients;
@@ -154,8 +144,8 @@ public:
 
     void timerCallback() override;
 
-
-    void updateFilterCoefficients(double sampleRate);
+    void updateFilterCoefficients (double sampleRate);
+    void calculateImageSourcePositions (const float t, const float b, const float h);
 
     float* numRefl;
     float mRadius[nImgSrc];
@@ -163,12 +153,11 @@ public:
     void updateBuffers() override;
 
     Atomic<bool> repaintPositionPlanes = true;
+
 private:
     //==============================================================================
-    OSCParameterInterface oscParams;
-    OSCReceiverPlus oscReceiver;
-    AudioProcessorValueTreeState parameters;
-    
+    inline void clear (AudioBlock<IIRfloat>& ab);
+
     bool readingSharedParams = false;;
 
     double phi;
@@ -205,23 +194,31 @@ private:
     float* syncListener;
 
     float* renderDirectPath;
+    float* directPathZeroDelay;
+    float* directPathUnityGain;
+
+    float* wallAttenuationFront;
+    float* wallAttenuationBack;
+    float* wallAttenuationLeft;
+    float* wallAttenuationRight;
+    float* wallAttenuationCeiling;
+    float* wallAttenuationFloor;
 
     int _numRefl;
 
     SharedResourcePointer<SharedParams> sharedParams;
 
     //SIMD IIR Filter
-    OwnedArray<IIR::Filter<IIRfloat>> lowShelfArray;
-    OwnedArray<IIR::Filter<IIRfloat>> highShelfArray;
-    OwnedArray<IIR::Filter<IIRfloat>> lowShelfArray2;
-    OwnedArray<IIR::Filter<IIRfloat>> highShelfArray2;
+
+    OwnedArray<OwnedArray<IIR::Filter<IIRfloat>>> lowShelfArray;
+    OwnedArray<OwnedArray<IIR::Filter<IIRfloat>>> highShelfArray;
     HeapBlock<char> interleavedBlockData[16], zeroData; //todo: dynamically?
     OwnedArray<AudioBlock<IIRfloat>> interleavedData;
     AudioBlock<float> zero;
 
+    Array<int> filterPoints {1, 7, 25, 61, 113, 169, 213};
 
     Vector3D<float> sourcePos, listenerPos;
-    float h,b,t;
 
     float mx[nImgSrc];
     float my[nImgSrc];
@@ -230,28 +227,21 @@ private:
     float smy[nImgSrc];
     float smz[nImgSrc];
 
-
-    float hypxy;
     int bufferSize;
     int bufferReadIdx;
-    int overflow;
-    int temp;
-
 
     int readOffset;
 
     float powReflCoeff[maxOrderImgSrc+1];
     double dist2smpls;
 
-    float *tempAddr;
     float SHcoeffsOld[nImgSrc][64];
     IIRfloat SHsampleOld[nImgSrc][16]; //TODO: can be smaller: (N+1)^2/IIRfloat_elements()
-    float weightedSample;
 
     AudioBuffer<float> delayBuffer;
     AudioBuffer<float> monoBuffer;
 
-    float** delayBufferWritePtrArray;
+    OwnedArray<ReflectionProperty> reflectionList;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (RoomEncoderAudioProcessor)
 };
