@@ -89,10 +89,10 @@ public:
             int numRef = roundToInt(*numReflPtr);
 
             float gainDb = Decibels::gainToDecibels(gainPtr[0]);
-            if (gainDb > -60.0f)
+            if (gainDb > -60.0f && gainDb <= 20.0f)
             {
-                float xPos = timeToX(radiusPtr[0]*xFactor);
-                float yPos = dBToY(gainDb);
+                const float xPos = timeToX (zeroDelay ? 0.0f : radiusPtr[0] * xFactor);
+                const float yPos = dBToY(gainDb);
                 g.drawLine(xPos, yPos, xPos, mT + plotHeight, 2.0f);
             }
 
@@ -101,11 +101,12 @@ public:
             for (int i = 1; i <= numRef; ++i)
             {
                 float gainDb = Decibels::gainToDecibels(gainPtr[i]);
-                if (gainDb > -60.0f)
+                if (gainDb > -60.0f && gainDb < 20.0f)
                 {
-                    float xPos = timeToX(radiusPtr[i]*xFactor);
-                    float yPos = dBToY(gainDb);
-                    g.drawLine(xPos, yPos, xPos, mT + plotHeight, 1.5f);
+                    const float radius = radiusPtr[i] - (zeroDelay ? radiusPtr[0] : 0.0f);
+                    const float xPos = timeToX (radius* xFactor);
+                    const float yPos = dBToY (gainDb);
+                    g.drawLine (xPos, yPos, xPos, mT + plotHeight, 1.5f);
                 }
             }
         }
@@ -134,6 +135,15 @@ public:
         radiusPtr = Radius;
     }
 
+    void setZeroDelay (const bool shouldBeZeroDelay)
+    {
+        if (zeroDelay != shouldBeZeroDelay)
+        {
+            zeroDelay = shouldBeZeroDelay;
+            repaint();
+        }
+    }
+
     void resized() override
     {
         plotWidth = getLocalBounds().getWidth() - mL - mR;
@@ -170,4 +180,6 @@ private:
     float* numReflPtr = nullptr;
     float* gainPtr = nullptr;
     float* radiusPtr = nullptr;
+
+    bool zeroDelay = false;
 };
