@@ -44,17 +44,18 @@
 
 //==============================================================================
 SceneRotatorAudioProcessor::SceneRotatorAudioProcessor()
+: AudioProcessorBase (
 #ifndef JucePlugin_PreferredChannelConfigurations
-: AudioProcessor (BusesProperties()
+                  BusesProperties()
 #if ! JucePlugin_IsMidiEffect
 #if ! JucePlugin_IsSynth
                   .withInput  ("Input",  AudioChannelSet::discreteChannels (64), true)
 #endif
                   .withOutput ("Output", AudioChannelSet::discreteChannels (64), true)
 #endif
-                  ),
+                  ,
 #endif
-oscParams (parameters), parameters (*this, nullptr, "SceneRotator", createParameterLayout())
+createParameterLayout())
 {
     // get pointers to the parameters
     orderSetting = parameters.getRawParameterValue ("orderSetting");
@@ -104,52 +105,15 @@ oscParams (parameters), parameters (*this, nullptr, "SceneRotator", createParame
         elemCopy->clear();
     }
 
-
-    oscReceiver.addListener (this);
+    startTimer (500);
 }
 
 SceneRotatorAudioProcessor::~SceneRotatorAudioProcessor()
 {
+    closeMidiInput();
 }
 
 //==============================================================================
-const String SceneRotatorAudioProcessor::getName() const
-{
-    return JucePlugin_Name;
-}
-
-bool SceneRotatorAudioProcessor::acceptsMidi() const
-{
-#if JucePlugin_WantsMidiInput
-    return true;
-#else
-    return false;
-#endif
-}
-
-bool SceneRotatorAudioProcessor::producesMidi() const
-{
-#if JucePlugin_ProducesMidiOutput
-    return true;
-#else
-    return false;
-#endif
-}
-
-bool SceneRotatorAudioProcessor::isMidiEffect() const
-{
-#if JucePlugin_IsMidiEffect
-    return true;
-#else
-    return false;
-#endif
-}
-
-double SceneRotatorAudioProcessor::getTailLengthSeconds() const
-{
-    return 0.0;
-}
-
 int SceneRotatorAudioProcessor::getNumPrograms()
 {
     return 1;   // NB: some hosts don't cope very well if you tell them there are 0 programs,
@@ -184,6 +148,7 @@ void SceneRotatorAudioProcessor::prepareToPlay (double sampleRate, int samplesPe
     // Use this method as the place to do any pre-playback
     // initialisation that you need..
 
+    MidiMessageCollector::reset (sampleRate);
     rotationParamsHaveChanged = true;
 
 }
@@ -194,13 +159,6 @@ void SceneRotatorAudioProcessor::releaseResources()
     // spare memory, etc.
 }
 
-#ifndef JucePlugin_PreferredChannelConfigurations
-bool SceneRotatorAudioProcessor::isBusesLayoutSupported (const BusesLayout& layouts) const
-{
-    return true;
-}
-#endif
-
 void SceneRotatorAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBuffer& midiMessages)
 {
     checkInputAndOutput (this, *orderSetting, *orderSetting, false);
@@ -214,6 +172,99 @@ void SceneRotatorAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBu
     const int actualChannels = square (actualOrder + 1);
     jassert (actualChannels <= nChIn);
 
+    if (currentMidiScheme != MidiScheme::none)
+    {
+        removeNextBlockOfMessages (midiMessages, buffer.getNumSamples());
+
+        MidiBuffer::Iterator i (midiMessages);
+        MidiMessage message;
+        int time;
+
+
+        while (i.getNextEvent (message, time))
+        {
+
+            if (! message.isController())
+                break;
+
+            switch (currentMidiScheme)
+            {
+                case MidiScheme::mrHeadTrackerYprDir:
+                case MidiScheme::mrHeadTrackerYprInv:
+                    switch (message.getControllerNumber())
+                    {
+                        case 48: yawLsb = message.getControllerValue(); break;
+                        case 49: pitchLsb = message.getControllerValue(); break;
+                        case 50: rollLsb = message.getControllerValue(); break;
+
+                        case 16:
+                        {
+                            float yawVal = (128 * message.getControllerValue() + yawLsb) * (1.0f / 16384);
+                            parameters.getParameter ("yaw")->setValueNotifyingHost (yawVal);
+                            break;
+                        }
+
+                        case 17:
+                        {
+                            float pitchVal = (128 * message.getControllerValue() + pitchLsb) * (1.0f / 16384);
+                            parameters.getParameter ("pitch")->setValueNotifyingHost (pitchVal);
+                            break;
+                        }
+
+                        case 18:
+                        {
+                            float rollVal = (128 * message.getControllerValue() + rollLsb) * (1.0f / 16384);
+                            parameters.getParameter ("roll")->setValueNotifyingHost (rollVal);
+                            break;
+                        }
+                    } // switch (message.getControllerNumber())
+                    break;
+
+                case MidiScheme::mrHeadTrackerQuaternions:
+                    switch (message.getControllerNumber())
+                    {
+                        case 48: qwLsb = message.getControllerValue(); break;
+                        case 49: qxLsb = message.getControllerValue(); break;
+                        case 50: qyLsb = message.getControllerValue(); break;
+                        case 51: qzLsb = message.getControllerValue(); break;
+
+                        case 16:
+                        {
+                            float qwVal = (128 * message.getControllerValue() + qwLsb) * (1.0f / 16384);
+                            parameters.getParameter ("qw")->setValueNotifyingHost (qwVal);
+                            break;
+                        }
+
+                        case 17:
+                        {
+                            float qxVal = (128 * message.getControllerValue() + qxLsb) * (1.0f / 16384);
+                            parameters.getParameter ("qx")->setValueNotifyingHost (qxVal);
+                            break;
+                        }
+
+                        case 18:
+                        {
+                            float qyVal = (128 * message.getControllerValue() + qyLsb) * (1.0f / 16384);
+                            parameters.getParameter ("qy")->setValueNotifyingHost (qyVal);
+                            break;
+                        }
+
+                        case 19:
+                        {
+                            float qzVal = (128 * message.getControllerValue() + qzLsb) * (1.0f / 16384);
+                            parameters.getParameter ("qz")->setValueNotifyingHost (qzVal);
+                            break;
+                        }
+                    } // switch (message.getControllerNumber())
+                    break;
+                default:
+                    break;
+            } // switch (currentMidiScheme)
+        } //while (i.getNextEvent (message, time))
+    } //if (currentMidiScheme != MidiScheme::none)
+
+
+
     bool newRotationMatrix = false;
     if (rotationParamsHaveChanged.get())
     {
@@ -251,6 +302,7 @@ void SceneRotatorAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBu
         for (int l = 1; l <= inputOrder; ++l)
             *orderMatricesCopy[l] = *orderMatrices[l];
 
+    midiMessages.clear();
 }
 
 double SceneRotatorAudioProcessor::P (int i, int l, int a, int b, Matrix<float>& R1, Matrix<float>& Rlm1)
@@ -430,11 +482,13 @@ AudioProcessorEditor* SceneRotatorAudioProcessor::createEditor()
 //==============================================================================
 void SceneRotatorAudioProcessor::getStateInformation (MemoryBlock& destData)
 {
-    // You should use this method to store your parameters in the memory block.
-    // You could do that either as raw data, or use the XML or ValueTree classes
-    // as intermediaries to make it easy to save and load complex data.
     auto state = parameters.copyState();
-    state.setProperty ("OSCPort", var (oscReceiver.getPortNumber()), nullptr);
+
+    auto oscConfig = state.getOrCreateChildWithName ("OSCConfig", nullptr);
+    oscConfig.copyPropertiesFrom (oscParameterInterface.getConfig(), nullptr);
+
+    state.setProperty ("MidiDeviceName", var (currentMidiDeviceName), nullptr);
+    state.setProperty ("MidiDeviceScheme", var (static_cast<int> (currentMidiScheme)), nullptr);
     std::unique_ptr<XmlElement> xml (state.createXml());
     copyXmlToBinary (*xml, destData);
 }
@@ -449,10 +503,23 @@ void SceneRotatorAudioProcessor::setStateInformation (const void* data, int size
         if (xmlState->hasTagName (parameters.state.getType()))
         {
             parameters.replaceState (ValueTree::fromXml (*xmlState));
-            if (parameters.state.hasProperty ("OSCPort"))
+            if (parameters.state.hasProperty ("OSCPort")) // legacy
             {
-                oscReceiver.connect (parameters.state.getProperty ("OSCPort", var (-1)));
+                oscParameterInterface.getOSCReceiver().connect (parameters.state.getProperty ("OSCPort", var (-1)));
+                parameters.state.removeProperty ("OSCPort", nullptr);
             }
+
+            auto oscConfig = parameters.state.getChildWithName ("OSCConfig");
+            if (oscConfig.isValid())
+                oscParameterInterface.setConfig (oscConfig);
+
+            if (parameters.state.hasProperty ("MidiDeviceName"))
+                openMidiInput (parameters.state.getProperty ("MidiDeviceName", var ("")), true);
+            else
+                closeMidiInput();
+
+            if (parameters.state.hasProperty ("MidiDeviceScheme"))
+                setMidiScheme (MidiScheme (static_cast<int> (parameters.state.getProperty ("MidiDeviceScheme", var (0)))));
         }
 
     usingYpr = true;
@@ -537,7 +604,7 @@ inline void SceneRotatorAudioProcessor::updateQuaternions()
         qz = -qz;
     }
 
-    
+
     updatingParams = true;
     parameters.getParameter ("qw")->setValueNotifyingHost (parameters.getParameterRange ("qw").convertTo0to1 (qw));
     parameters.getParameter ("qx")->setValueNotifyingHost (parameters.getParameterRange ("qx").convertTo0to1 (qx));
@@ -617,153 +684,225 @@ void SceneRotatorAudioProcessor::updateBuffers()
     copyBuffer.setSize (input.getNumberOfChannels(), copyBuffer.getNumSamples());
 }
 
-//==============================================================================
-pointer_sized_int SceneRotatorAudioProcessor::handleVstPluginCanDo (int32 index,
-                                                                    pointer_sized_int value, void* ptr, float opt)
-{
-    auto text = (const char*) ptr;
-    auto matches = [=](const char* s) { return strcmp (text, s) == 0; };
-
-    if (matches ("wantsChannelCountNotifications"))
-        return 1;
-    return 0;
-}
 
 //==============================================================================
-void SceneRotatorAudioProcessor::oscMessageReceived (const OSCMessage &message)
+const bool SceneRotatorAudioProcessor::interceptOSCMessage (OSCMessage &message)
 {
     String prefix ("/" + String (JucePlugin_Name));
-    if (! message.getAddressPattern().toString().startsWith (prefix))
-        return;
-
-    OSCMessage msg (message);
-    msg.setAddressPattern (message.getAddressPattern().toString().substring (String (JucePlugin_Name).length() + 1));
-
-    if (! oscParams.processOSCMessage (msg))
+    if (message.getAddressPattern().toString().equalsIgnoreCase ("/" + String (JucePlugin_Name) + "/quaternions") && message.size() == 4)
     {
-        if (msg.getAddressPattern().toString().equalsIgnoreCase ("/quaternions") && msg.size() == 4)
-        {
-            float qs[4];
-            for (int i = 0; i < 4; ++i)
-                if (msg[i].isFloat32())
-                    qs[i] = msg[i].getFloat32();
-                else if (msg[i].isInt32())
-                    qs[i] = msg[i].getInt32();
+        float qs[4];
+        for (int i = 0; i < 4; ++i)
+            if (message[i].isFloat32())
+                qs[i] = message[i].getFloat32();
+            else if (message[i].isInt32())
+                qs[i] = message[i].getInt32();
 
-            oscParams.setValue ("qw", qs[0]);
-            oscParams.setValue ("qx", qs[1]);
-            oscParams.setValue ("qy", qs[2]);
-            oscParams.setValue ("qz", qs[3]);
-        }
-        else if (msg.getAddressPattern().toString().equalsIgnoreCase ("/ypr") && msg.size() == 3)
-        {
-            float ypr[3];
-            for (int i = 0; i < 3; ++i)
-                if (msg[i].isFloat32())
-                    ypr[i] = msg[i].getFloat32();
-                else if (msg[i].isInt32())
-                    ypr[i] = msg[i].getInt32();
-
-            oscParams.setValue ("yaw", ypr[0]);
-            oscParams.setValue ("pitch", ypr[1]);
-            oscParams.setValue ("roll", ypr[2]);
-        }
+        oscParameterInterface.setValue ("qw", qs[0]);
+        oscParameterInterface.setValue ("qx", qs[1]);
+        oscParameterInterface.setValue ("qy", qs[2]);
+        oscParameterInterface.setValue ("qz", qs[3]);
+        return true;
     }
+    else if (message.getAddressPattern().toString().equalsIgnoreCase ("/" + String (JucePlugin_Name) + "/ypr") && message.size() == 3)
+    {
+        float ypr[3];
+        for (int i = 0; i < 3; ++i)
+            if (message[i].isFloat32())
+                ypr[i] = message[i].getFloat32();
+            else if (message[i].isInt32())
+                ypr[i] = message[i].getInt32();
+
+        oscParameterInterface.setValue ("yaw", ypr[0]);
+        oscParameterInterface.setValue ("pitch", ypr[1]);
+        oscParameterInterface.setValue ("roll", ypr[2]);
+        return true;
+    }
+
+    return false;
 }
 
-void SceneRotatorAudioProcessor::oscBundleReceived (const OSCBundle &bundle)
-{
-    for (int i = 0; i < bundle.size(); ++i)
-    {
-        auto elem = bundle[i];
-        if (elem.isMessage())
-            oscMessageReceived (elem.getMessage());
-        else if (elem.isBundle())
-            oscBundleReceived (elem.getBundle());
-    }
-}
+
 
 //==============================================================================
-AudioProcessorValueTreeState::ParameterLayout SceneRotatorAudioProcessor::createParameterLayout()
+std::vector<std::unique_ptr<RangedAudioParameter>> SceneRotatorAudioProcessor::createParameterLayout()
 {
     // add your audio parameters here
     std::vector<std::unique_ptr<RangedAudioParameter>> params;
 
 
-    params.push_back (oscParams.createAndAddParameter ("orderSetting", "Ambisonics Order", "",
-                                     NormalisableRange<float> (0.0f, 8.0f, 1.0f), 0.0f,
-                                     [](float value) {
-                                         if (value >= 0.5f && value < 1.5f) return "0th";
-                                         else if (value >= 1.5f && value < 2.5f) return "1st";
-                                         else if (value >= 2.5f && value < 3.5f) return "2nd";
-                                         else if (value >= 3.5f && value < 4.5f) return "3rd";
-                                         else if (value >= 4.5f && value < 5.5f) return "4th";
-                                         else if (value >= 5.5f && value < 6.5f) return "5th";
-                                         else if (value >= 6.5f && value < 7.5f) return "6th";
-                                         else if (value >= 7.5f) return "7th";
-                                         else return "Auto";
-                                     }, nullptr));
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("orderSetting", "Ambisonics Order", "",
+                                                       NormalisableRange<float> (0.0f, 8.0f, 1.0f), 0.0f,
+                                                       [](float value) {
+                                                           if (value >= 0.5f && value < 1.5f) return "0th";
+                                                           else if (value >= 1.5f && value < 2.5f) return "1st";
+                                                           else if (value >= 2.5f && value < 3.5f) return "2nd";
+                                                           else if (value >= 3.5f && value < 4.5f) return "3rd";
+                                                           else if (value >= 4.5f && value < 5.5f) return "4th";
+                                                           else if (value >= 5.5f && value < 6.5f) return "5th";
+                                                           else if (value >= 6.5f && value < 7.5f) return "6th";
+                                                           else if (value >= 7.5f) return "7th";
+                                                           else return "Auto";
+                                                       }, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("useSN3D", "Normalization", "",
-                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
-                                     [](float value)
-                                     {
-                                         if (value >= 0.5f ) return "SN3D";
-                                         else return "N3D";
-                                     }, nullptr));
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("useSN3D", "Normalization", "",
+                                                       NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
+                                                       [](float value)
+                                                       {
+                                                           if (value >= 0.5f ) return "SN3D";
+                                                           else return "N3D";
+                                                       }, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("yaw", "Yaw Angle", CharPointer_UTF8 (R"(°)"),
-                                     NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
-                                     [](float value) { return String(value, 2); }, nullptr, true));
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("yaw", "Yaw Angle", CharPointer_UTF8 (R"(°)"),
+                                                       NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
+                                                       [](float value) { return String(value, 2); }, nullptr, true));
 
-    params.push_back (oscParams.createAndAddParameter ("pitch", "Pitch Angle", CharPointer_UTF8 (R"(°)"),
-                                     NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
-                                     [](float value) { return String(value, 2); }, nullptr, true));
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("pitch", "Pitch Angle", CharPointer_UTF8 (R"(°)"),
+                                                       NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
+                                                       [](float value) { return String(value, 2); }, nullptr, true));
 
-    params.push_back (oscParams.createAndAddParameter ("roll", "Roll Angle", CharPointer_UTF8 (R"(°)"),
-                                     NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
-                                     [](float value) { return String(value, 2); }, nullptr, true));
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("roll", "Roll Angle", CharPointer_UTF8 (R"(°)"),
+                                                       NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
+                                                       [](float value) { return String(value, 2); }, nullptr, true));
 
-    params.push_back (oscParams.createAndAddParameter ("qw", "Quaternion W", "",
-                                     NormalisableRange<float> (-1.0f, 1.0f, 0.001f), 1.0,
-                                     [](float value) { return String(value, 2); }, nullptr, true));
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("qw", "Quaternion W", "",
+                                                       NormalisableRange<float> (-1.0f, 1.0f, 0.001f), 1.0,
+                                                       [](float value) { return String(value, 2); }, nullptr, true));
 
-    params.push_back (oscParams.createAndAddParameter ("qx", "Quaternion X", "",
-                                     NormalisableRange<float> (-1.0f, 1.0f, 0.001f), 0.0,
-                                     [](float value) { return String(value, 2); }, nullptr, true));
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("qx", "Quaternion X", "",
+                                                       NormalisableRange<float> (-1.0f, 1.0f, 0.001f), 0.0,
+                                                       [](float value) { return String(value, 2); }, nullptr, true));
 
-    params.push_back (oscParams.createAndAddParameter ("qy", "Quaternion Y", "",
-                                     NormalisableRange<float> (-1.0f, 1.0f, 0.001f), 0.0,
-                                     [](float value) { return String(value, 2); }, nullptr, true));
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("qy", "Quaternion Y", "",
+                                                       NormalisableRange<float> (-1.0f, 1.0f, 0.001f), 0.0,
+                                                       [](float value) { return String(value, 2); }, nullptr, true));
 
-    params.push_back (oscParams.createAndAddParameter ("qz", "Quaternion Z", "",
-                                     NormalisableRange<float> (-1.0f, 1.0f, 0.001f), 0.0,
-                                     [](float value) { return String(value, 2); }, nullptr, true));
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("qz", "Quaternion Z", "",
+                                                       NormalisableRange<float> (-1.0f, 1.0f, 0.001f), 0.0,
+                                                       [](float value) { return String(value, 2); }, nullptr, true));
 
-    params.push_back (oscParams.createAndAddParameter ("invertYaw", "Invert Yaw", "",
-                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0,
-                                     [](float value) { return value >= 0.5f ? "ON" : "OFF"; }, nullptr));
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("invertYaw", "Invert Yaw", "",
+                                                       NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0,
+                                                       [](float value) { return value >= 0.5f ? "ON" : "OFF"; }, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("invertPitch", "Invert Pitch", "",
-                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0,
-                                     [](float value) { return value >= 0.5f ? "ON" : "OFF"; }, nullptr));
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("invertPitch", "Invert Pitch", "",
+                                                       NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0,
+                                                       [](float value) { return value >= 0.5f ? "ON" : "OFF"; }, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("invertRoll", "Invert Roll", "",
-                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0,
-                                     [](float value) { return value >= 0.5f ? "ON" : "OFF"; }, nullptr));
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("invertRoll", "Invert Roll", "",
+                                                       NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0,
+                                                       [](float value) { return value >= 0.5f ? "ON" : "OFF"; }, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("invertQuaternion", "Invert Quaternion", "",
-                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0,
-                                     [](float value) { return value >= 0.5f ? "ON" : "OFF"; }, nullptr));
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("invertQuaternion", "Invert Quaternion", "",
+                                                       NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0,
+                                                       [](float value) { return value >= 0.5f ? "ON" : "OFF"; }, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("rotationSequence", "Sequence of Rotations", "",
-                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0,
-                                     [](float value) { return value >= 0.5f ? "Roll->Pitch->Yaw" : "Yaw->Pitch->Roll"; }, nullptr));
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("rotationSequence", "Sequence of Rotations", "",
+                                                       NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0,
+                                                       [](float value) { return value >= 0.5f ? "Roll->Pitch->Yaw" : "Yaw->Pitch->Roll"; }, nullptr));
 
 
+    return params;
+}
+
+//==============================================================================
+void SceneRotatorAudioProcessor::timerCallback()
+{
+    // retrying to connect to a desired device which might not be physically connected
+    if (currentMidiDeviceName != "" && midiInput == nullptr)
+        openMidiInput (currentMidiDeviceName);
+}
 
 
-    return { params.begin(), params.end() };
+//==============================================================================
+String SceneRotatorAudioProcessor::getCurrentMidiDeviceName()
+{
+    return currentMidiDeviceName;
+}
+
+void SceneRotatorAudioProcessor::openMidiInput (String midiDeviceName, bool forceUpdatingCurrentMidiDeviceName)
+{
+    if (midiDeviceName.isEmpty())
+        return closeMidiInput(); // <- not sure if that syntax is totally wrong or brilliant!
+
+    const ScopedLock scopedLock (changingMidiDevice);
+
+    StringArray devices = MidiInput::getDevices();
+
+    const int index = devices.indexOf (midiDeviceName);
+    if (index != -1)
+    {
+        midiInput = MidiInput::openDevice (index, this);
+        if (midiInput == nullptr)
+        {
+            deviceHasChanged = true;
+            showMidiOpenError = true;
+            return;
+        }
+
+        midiInput->start();
+
+        DBG ("Opened MidiInput: " << midiInput->getName());
+
+        currentMidiDeviceName = midiDeviceName;
+        deviceHasChanged = true;
+
+        return;
+    }
+
+    if (forceUpdatingCurrentMidiDeviceName)
+    {
+        currentMidiDeviceName = midiDeviceName;
+        deviceHasChanged = true;
+    }
+
+    return;
+}
+
+void SceneRotatorAudioProcessor::closeMidiInput()
+{
+    const ScopedLock scopedLock (changingMidiDevice);
+    if (midiInput != nullptr)
+    {
+        midiInput->stop();
+        midiInput.reset();
+        DBG ("Closed MidiInput");
+    }
+
+    currentMidiDeviceName = ""; // hoping there's not actually a MidiDevice without a name!
+    deviceHasChanged = true;
+
+    return;
+}
+
+void SceneRotatorAudioProcessor::setMidiScheme (MidiScheme newMidiScheme)
+{
+    currentMidiScheme = newMidiScheme;
+    DBG ("Scheme set to " << midiSchemeNames[static_cast<int> (newMidiScheme)]);
+
+    switch (newMidiScheme)
+    {
+        case MidiScheme::none:
+            break;
+
+        case MidiScheme::mrHeadTrackerYprDir:
+            parameters.getParameter ("rotationSequence")->setValueNotifyingHost (1.0f); // roll->pitch->yaw
+            break;
+
+        case MidiScheme::mrHeadTrackerYprInv:
+            parameters.getParameter ("rotationSequence")->setValueNotifyingHost (1.0f); // roll->pitch->yaw
+            break;
+
+        case MidiScheme::mrHeadTrackerQuaternions:
+            break;
+
+        default:
+            DBG ("Not supported MidiScheme - I guess the casting from int failed hard!");
+            jassertfalse;
+            break;
+    }
+
+    schemeHasChanged = true;
 }
 
 //==============================================================================
