@@ -23,20 +23,22 @@
 #include "PluginProcessor.h"
 #include "PluginEditor.h"
 
+const StringArray SimpleDecoderAudioProcessor::weightsStrings =  StringArray ("basic", "maxrE", "inphase");
 
 //==============================================================================
 SimpleDecoderAudioProcessor::SimpleDecoderAudioProcessor()
+: AudioProcessorBase (
 #ifndef JucePlugin_PreferredChannelConfigurations
-: AudioProcessor (BusesProperties()
+                  BusesProperties()
 #if ! JucePlugin_IsMidiEffect
 #if ! JucePlugin_IsSynth
                   .withInput  ("Input",  AudioChannelSet::discreteChannels(64), true)
 #endif
                   .withOutput ("Output", AudioChannelSet::discreteChannels(64), true)
 #endif
-                  ),
+                  ,
 #endif
-oscParams (parameters), parameters(*this, nullptr, "Decoder", createParameterLayout())
+createParameterLayout())
 {
     // dummy values
     cascadedLowPassCoeffs = IIR::Coefficients<double>::makeLowPass(48000.0, 100.0f);
@@ -56,6 +58,7 @@ oscParams (parameters), parameters(*this, nullptr, "Decoder", createParameterLay
 
     swMode = parameters.getRawParameterValue ("swMode");
     swChannel = parameters.getRawParameterValue("swChannel");
+    weights = parameters.getRawParameterValue ("weights");
 
     // add listeners to parameter changes
 
@@ -69,8 +72,7 @@ oscParams (parameters), parameters(*this, nullptr, "Decoder", createParameterLay
     parameters.addParameterListener ("highPassQ", this);
 
     parameters.addParameterListener ("swMode", this);
-
-
+    parameters.addParameterListener ("weights", this);
 
     highPassSpecs.numChannels = 0;
 
@@ -81,7 +83,7 @@ oscParams (parameters), parameters(*this, nullptr, "Decoder", createParameterLay
     options.folderName          = "IEM";
     options.osxLibrarySubFolder = "Preferences";
 
-    properties = new PropertiesFile(options);
+    properties.reset (new PropertiesFile (options));
     lastDir = File(properties->getValue("presetFolder"));
 
 
@@ -89,10 +91,8 @@ oscParams (parameters), parameters(*this, nullptr, "Decoder", createParameterLay
     highPass1.state = highPassCoeffs;
     highPass2.state = highPassCoeffs;
 
-    lowPass1 = new IIR::Filter<float>(lowPassCoeffs);
-    lowPass2 = new IIR::Filter<float>(lowPassCoeffs);
-
-    oscReceiver.addListener (this);
+    lowPass1.reset (new IIR::Filter<float> (lowPassCoeffs));
+    lowPass2.reset (new IIR::Filter<float> (lowPassCoeffs));
 }
 
 SimpleDecoderAudioProcessor::~SimpleDecoderAudioProcessor()
@@ -101,6 +101,7 @@ SimpleDecoderAudioProcessor::~SimpleDecoderAudioProcessor()
 
 void SimpleDecoderAudioProcessor::updateLowPassCoefficients (double sampleRate, float frequency)
 {
+    frequency = jmin (static_cast<float> (0.5 * sampleRate), frequency);
     *lowPassCoeffs = *IIR::Coefficients<float>::makeLowPass (sampleRate, frequency);
 
     auto newCoeffs = IIR::Coefficients<double>::makeLowPass (sampleRate, frequency);
@@ -111,6 +112,7 @@ void SimpleDecoderAudioProcessor::updateLowPassCoefficients (double sampleRate,
 
 void SimpleDecoderAudioProcessor::updateHighPassCoefficients(double sampleRate, float frequency)
 {
+    frequency = jmin (static_cast<float> (0.5 * sampleRate), frequency);
     *highPassCoeffs = *IIR::Coefficients<float>::makeHighPass (sampleRate, frequency);
 
     auto newCoeffs = IIR::Coefficients<double>::makeHighPass (sampleRate, frequency);
@@ -129,43 +131,6 @@ void SimpleDecoderAudioProcessor::setLastDir(File newLastDir)
 
 
 //==============================================================================
-const String SimpleDecoderAudioProcessor::getName() const
-{
-    return JucePlugin_Name;
-}
-
-bool SimpleDecoderAudioProcessor::acceptsMidi() const
-{
-#if JucePlugin_WantsMidiInput
-    return true;
-#else
-    return false;
-#endif
-}
-
-bool SimpleDecoderAudioProcessor::producesMidi() const
-{
-#if JucePlugin_ProducesMidiOutput
-    return true;
-#else
-    return false;
-#endif
-}
-
-bool SimpleDecoderAudioProcessor::isMidiEffect() const
-{
-#if JucePlugin_IsMidiEffect
-    return true;
-#else
-    return false;
-#endif
-}
-
-double SimpleDecoderAudioProcessor::getTailLengthSeconds() const
-{
-    return 0.0;
-}
-
 int SimpleDecoderAudioProcessor::getNumPrograms()
 {
     return 1;   // NB: some hosts don't cope very well if you tell them there are 0 programs,
@@ -237,7 +202,12 @@ void SimpleDecoderAudioProcessor::prepareToPlay (double sampleRate, int samplesP
     lowPass2->prepare(highPassSpecs);
     lowPass2->reset();
 
+    masterGain.setRampDurationSeconds (0.1f);
+    masterGain.prepare ({sampleRate, static_cast<uint32> (samplesPerBlock), 1});
+
     decoder.setInputNormalization(*useSN3D >= 0.5f ? ReferenceCountedDecoder::Normalization::sn3d : ReferenceCountedDecoder::Normalization::n3d);
+
+    guiUpdateSampleRate = true;
 }
 
 
@@ -247,12 +217,6 @@ void SimpleDecoderAudioProcessor::releaseResources()
     // spare memory, etc.
 }
 
-#ifndef JucePlugin_PreferredChannelConfigurations
-bool SimpleDecoderAudioProcessor::isBusesLayoutSupported (const BusesLayout& layouts) const
-{
-    return true;
-}
-#endif
 
 void SimpleDecoderAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBuffer& midiMessages)
 {
@@ -321,11 +285,16 @@ void SimpleDecoderAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiB
         highPass2.process(highPassContext);
     }
 
+    // update current weights setting
+    auto settings = retainedDecoder->getSettings();
+    settings.weights = ReferenceCountedDecoder::Weights (roundToInt (*weights));
+    retainedDecoder->setSettings (settings);
+
+    // ambisonic decoding
     const int L = buffer.getNumSamples();
-    AudioBlock<float> inputAudioBlock = AudioBlock<float>(buffer.getArrayOfWritePointers(), nChIn, L);
-    AudioBlock<float> outputAudioBlock = AudioBlock<float>(buffer.getArrayOfWritePointers(), nChOut, L);
-    ProcessContextNonReplacing<float> decoderContext (inputAudioBlock, outputAudioBlock);
-    decoder.process(decoderContext);
+    auto inputAudioBlock = AudioBlock<float> (buffer.getArrayOfWritePointers(), nChIn, L);
+    auto outputAudioBlock = AudioBlock<float> (buffer.getArrayOfWritePointers(), nChOut, L);
+    decoder.process (inputAudioBlock, outputAudioBlock);
 
     for (int ch = nChOut; ch < nChIn; ++ch) // clear all not needed channels
         buffer.clear(ch, 0, buffer.getNumSamples());
@@ -349,7 +318,12 @@ void SimpleDecoderAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiB
                 buffer.addFrom(destCh, 0, swBuffer, 0, 0, buffer.getNumSamples());
         }
     }
-    // ======================================================================
+    // =================== Master Gain =========================================
+    auto overallGainInDecibels = *parameters.getRawParameterValue ("overallGain");
+    masterGain.setGainDecibels (overallGainInDecibels);
+    AudioBlock<float> ab (buffer.getArrayOfWritePointers(), nChOut,  buffer.getNumSamples());
+    ProcessContextReplacing<float> masterContext (ab);
+    masterGain.process (masterContext);
 }
 
 //==============================================================================
@@ -366,12 +340,15 @@ AudioProcessorEditor* SimpleDecoderAudioProcessor::createEditor()
 //==============================================================================
 void SimpleDecoderAudioProcessor::getStateInformation (MemoryBlock& destData)
 {
-    // You should use this method to store your parameters in the memory block.
-    // You could do that either as raw data, or use the XML or ValueTree classes
-    // as intermediaries to make it easy to save and load complex data.
-    parameters.state.setProperty("lastOpenedPresetFile", var(lastFile.getFullPathName()), nullptr);
-    parameters.state.setProperty ("OSCPort", var(oscReceiver.getPortNumber()), nullptr);
-    ScopedPointer<XmlElement> xml (parameters.state.createXml());
+    auto state = parameters.copyState();
+
+    state.setProperty ("lastOpenedPresetFile", var (lastFile.getFullPathName()), nullptr);
+
+    auto oscConfig = state.getOrCreateChildWithName ("OSCConfig", nullptr);
+    oscConfig.copyPropertiesFrom (oscParameterInterface.getConfig(), nullptr);
+
+    std::unique_ptr<XmlElement> xml (state.createXml());
+    xml->setTagName (String (JucePlugin_Name)); // converts old "Decoder" state to "SimpleDecoder" state
     copyXmlToBinary (*xml, destData);
 }
 
@@ -381,23 +358,33 @@ void SimpleDecoderAudioProcessor::setStateInformation (const void* data, int siz
 {
     // You should use this method to restore your parameters from this memory block,
     // whose contents will have been created by the getStateInformation() call.
-    ScopedPointer<XmlElement> xmlState (getXmlFromBinary (data, sizeInBytes));
+    std::unique_ptr<XmlElement> xmlState (getXmlFromBinary (data, sizeInBytes));
     if (xmlState != nullptr)
-        if (xmlState->hasTagName (parameters.state.getType()))
+        if (xmlState->hasTagName (parameters.state.getType()) || xmlState->hasTagName ("Decoder")) // compatibility for old "Decoder" state tagName
             parameters.state = ValueTree::fromXml (*xmlState);
-    if (parameters.state.hasProperty("lastOpenedPresetFile"))
+    if (parameters.state.hasProperty ("lastOpenedPresetFile"))
     {
-        Value val = parameters.state.getPropertyAsValue("lastOpenedPresetFile", nullptr);
+        Value val = parameters.state.getPropertyAsValue ("lastOpenedPresetFile", nullptr);
         if (val.getValue().toString() != "")
         {
+            auto* weightsParam = parameters.getParameter ("weights");
+            const auto savedWeights = weightsParam->getValue();
             const File f (val.getValue().toString());
             loadConfiguration(f);
-        }
-        if (parameters.state.hasProperty ("OSCPort"))
-        {
-            oscReceiver.connect (parameters.state.getProperty ("OSCPort", var (-1)));
+            weightsParam->setValueNotifyingHost (savedWeights);
         }
     }
+
+    if (parameters.state.hasProperty ("OSCPort")) // legacy
+    {
+        oscParameterInterface.getOSCReceiver().connect (parameters.state.getProperty ("OSCPort", var (-1)));
+        parameters.state.removeProperty ("OSCPort", nullptr);
+    }
+
+    auto oscConfig = parameters.state.getChildWithName ("OSCConfig");
+    if (oscConfig.isValid())
+        oscParameterInterface.setConfig (oscConfig);
+
 }
 
 //==============================================================================
@@ -434,78 +421,50 @@ void SimpleDecoderAudioProcessor::loadConfiguration (const File& presetFile)
     ReferenceCountedDecoder::Ptr tempDecoder = nullptr;
 
     Result result = ConfigurationHelper::parseFileForDecoder (presetFile, &tempDecoder);
-    if (!result.wasOk()) {
+    if (result.failed())
         messageForEditor = result.getErrorMessage();
-    }
 
     if (tempDecoder != nullptr)
     {
         lastFile = presetFile;
         messageForEditor = "";
+
+        tempDecoder->removeAppliedWeights();
+        parameters.getParameterAsValue ("weights").setValue (static_cast<int> (tempDecoder->getSettings().weights));
     }
 
-    decoder.setDecoder(tempDecoder);
+
+    decoder.setDecoder (tempDecoder);
     decoderConfig = tempDecoder;
 
     updateDecoderInfo = true;
     messageChanged = true;
 }
 
-//==============================================================================
-pointer_sized_int SimpleDecoderAudioProcessor::handleVstPluginCanDo (int32 index,
-                                                                     pointer_sized_int value, void* ptr, float opt)
-{
-    auto text = (const char*) ptr;
-    auto matches = [=](const char* s) { return strcmp (text, s) == 0; };
-
-    if (matches ("wantsChannelCountNotifications"))
-        return 1;
-    return 0;
-}
 
 //==============================================================================
-void SimpleDecoderAudioProcessor::oscMessageReceived (const OSCMessage &message)
+const bool SimpleDecoderAudioProcessor::processNotYetConsumedOSCMessage (const OSCMessage &message)
 {
-    String prefix ("/" + String(JucePlugin_Name));
-    if (! message.getAddressPattern().toString().startsWith (prefix))
-        return;
-
-    OSCMessage msg (message);
-    msg.setAddressPattern (message.getAddressPattern().toString().substring(String(JucePlugin_Name).length() + 1));
-
-    if (! oscParams.processOSCMessage (msg))
+    if (message.getAddressPattern().toString().equalsIgnoreCase ("/" + String (JucePlugin_Name) + "/loadFile") && message.size() >= 1)
     {
-        // Load configuration file
-        if (msg.getAddressPattern().toString().equalsIgnoreCase("/loadFile") && msg.size() >= 1)
+        if (message[0].isString())
         {
-            if (msg[0].isString())
-            {
-                File fileToLoad (msg[0].getString());
-                loadConfiguration (fileToLoad);
-            }
+            File fileToLoad (message[0].getString());
+            loadConfiguration (fileToLoad);
+            return true;
         }
     }
-}
 
-void SimpleDecoderAudioProcessor::oscBundleReceived (const OSCBundle &bundle)
-{
-    for (int i = 0; i < bundle.size(); ++i)
-    {
-        auto elem = bundle[i];
-        if (elem.isMessage())
-            oscMessageReceived (elem.getMessage());
-        else if (elem.isBundle())
-            oscBundleReceived (elem.getBundle());
-    }
+    return false;
 }
 
 //==============================================================================
-AudioProcessorValueTreeState::ParameterLayout SimpleDecoderAudioProcessor::createParameterLayout()
+std::vector<std::unique_ptr<RangedAudioParameter>> SimpleDecoderAudioProcessor::createParameterLayout()
 {
     // add your audio parameters here
     std::vector<std::unique_ptr<RangedAudioParameter>> params;
 
-    params.push_back (oscParams.createAndAddParameter ("inputOrderSetting", "Ambisonic Order", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("inputOrderSetting", "Ambisonic Order", "",
                                      NormalisableRange<float> (0.0f, 8.0f, 1.0f), 0.0f,
                                      [](float value) {
                                          if (value >= 0.5f && value < 1.5f) return "0th";
@@ -519,41 +478,44 @@ AudioProcessorValueTreeState::ParameterLayout SimpleDecoderAudioProcessor::creat
                                          else return "Auto";},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter("useSN3D", "Normalization", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay("useSN3D", "Normalization", "",
                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
                                     [](float value) {
                                         if (value >= 0.5f) return "SN3D";
                                         else return "N3D";
                                     }, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("lowPassFrequency", "LowPass Cutoff Frequency", "Hz",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("lowPassFrequency", "LowPass Cutoff Frequency", "Hz",
                                      NormalisableRange<float> (20.f, 300.f, 1.0f), 80.f,
                                      [](float value) {return String ((int) value);},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("lowPassGain", "LowPass Gain", "dB",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("lowPassGain", "LowPass Gain", "dB",
                                      NormalisableRange<float> (-20.0f, 10.0, 0.1f), 0.0f,
                                      [](float value) {return String (value, 1);},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("highPassFrequency", "HighPass Cutoff Frequency", "Hz",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("highPassFrequency", "HighPass Cutoff Frequency", "Hz",
                                      NormalisableRange<float> (20.f, 300.f, 1.f), 80.f,
                                      [](float value) {return String ((int) value);},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("swMode", "Subwoofer Mode", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("swMode", "Subwoofer Mode", "",
                                      NormalisableRange<float> (0.0f, 2.0f, 1.0f), 0.0f,
                                      [](float value) {
                                          if (value < 0.5f) return "none";
                                          else if (value >= 0.5f && value < 1.5f) return "Discrete SW";
                                          else return "Virtual SW";}, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("swChannel", "SW Channel Number", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("swChannel", "SW Channel Number", "",
                                      NormalisableRange<float> (1.0f, 64.0f, 1.0f), 1.0f,
                                      [](float value) { return String ((int) value);}, nullptr));
 
+    params.push_back (std::make_unique<AudioParameterChoice> ("weights", "Ambisonic Weights", weightsStrings, 1));
 
-    return { params.begin(), params.end() };
+    params.push_back (std::make_unique<AudioParameterFloat> ("overallGain", "Overall Gain", NormalisableRange<float> (-20.0f, 20.0f, 0.01f), 0.0f, "dB", AudioProcessorParameter::outputGain, [] (float value, int maximumStringLength) { return String (value, maximumStringLength); }, nullptr));
+
+    return params;
 }
 
 //==============================================================================
