@@ -26,27 +26,29 @@
 
 //==============================================================================
 ToolBoxAudioProcessor::ToolBoxAudioProcessor()
+: AudioProcessorBase (
 #ifndef JucePlugin_PreferredChannelConfigurations
-     : AudioProcessor (BusesProperties()
+                       BusesProperties()
                      #if ! JucePlugin_IsMidiEffect
                       #if ! JucePlugin_IsSynth
                        .withInput  ("Input",  AudioChannelSet::discreteChannels(64), true)
                       #endif
                        .withOutput ("Output", AudioChannelSet::discreteChannels(64), true)
                      #endif
-                       ),
+                       ,
 #endif
-oscParams (parameters), parameters (*this, nullptr, "ToolBox", createParameterLayout()), flipXMask (int64 (0)), flipYMask (int64 (0)), flipZMask (int64 (0))
+createParameterLayout()), flipXMask (int64 (0)), flipYMask (int64 (0)), flipZMask (int64 (0))
 {
     // get pointers to the parameters
-    inputOrderSetting = parameters.getRawParameterValue("inputOrderSetting");
+    inputOrderSetting = parameters.getRawParameterValue ("inputOrderSetting");
     outputOrderSetting = parameters.getRawParameterValue ("outputOrderSetting");
     useSn3dInput = parameters.getRawParameterValue ("useSn3dInput");
     useSn3dOutput = parameters.getRawParameterValue ("useSn3dOutput");
     flipX = parameters.getRawParameterValue ("flipX");
     flipY = parameters.getRawParameterValue ("flipY");
     flipZ = parameters.getRawParameterValue ("flipZ");
-    loaWeights = parameters.getRawParameterValue("loaWeights");
+    loaWeights = parameters.getRawParameterValue ("loaWeights");
+    gain = parameters.getRawParameterValue ("gain");
 
 
     // add listeners to parameter changes
@@ -74,8 +76,6 @@ oscParams (parameters), parameters (*this, nullptr, "ToolBox", createParameterLa
         if ((l + m) % 2)
             flipZMask.setBit(ch);
     }
-
-    oscReceiver.addListener (this);
 }
 
 ToolBoxAudioProcessor::~ToolBoxAudioProcessor()
@@ -83,43 +83,6 @@ ToolBoxAudioProcessor::~ToolBoxAudioProcessor()
 }
 
 //==============================================================================
-const String ToolBoxAudioProcessor::getName() const
-{
-    return JucePlugin_Name;
-}
-
-bool ToolBoxAudioProcessor::acceptsMidi() const
-{
-   #if JucePlugin_WantsMidiInput
-    return true;
-   #else
-    return false;
-   #endif
-}
-
-bool ToolBoxAudioProcessor::producesMidi() const
-{
-   #if JucePlugin_ProducesMidiOutput
-    return true;
-   #else
-    return false;
-   #endif
-}
-
-bool ToolBoxAudioProcessor::isMidiEffect() const
-{
-   #if JucePlugin_IsMidiEffect
-    return true;
-   #else
-    return false;
-   #endif
-}
-
-double ToolBoxAudioProcessor::getTailLengthSeconds() const
-{
-    return 0.0;
-}
-
 int ToolBoxAudioProcessor::getNumPrograms()
 {
     return 1;   // NB: some hosts don't cope very well if you tell them there are 0 programs,
@@ -147,11 +110,13 @@ void ToolBoxAudioProcessor::changeProgramName (int index, const String& newName)
 //==============================================================================
 void ToolBoxAudioProcessor::prepareToPlay (double sampleRate, int samplesPerBlock)
 {
-    checkInputAndOutput(this, *inputOrderSetting, *outputOrderSetting, true);
+    checkInputAndOutput (this, *inputOrderSetting, *outputOrderSetting, true);
 
     doFlipX = *flipX >= 0.5f;
     doFlipY = *flipY >= 0.5f;
     doFlipZ = *flipZ >= 0.5f;
+
+    calculateWeights (previousWeights, input.getNumberOfChannels(), output.getNumberOfChannels());
 }
 
 void ToolBoxAudioProcessor::releaseResources()
@@ -160,29 +125,51 @@ void ToolBoxAudioProcessor::releaseResources()
     // spare memory, etc.
 }
 
-#ifndef JucePlugin_PreferredChannelConfigurations
-bool ToolBoxAudioProcessor::isBusesLayoutSupported (const BusesLayout& layouts) const
-{
-    return true;
-}
-#endif
 
 void ToolBoxAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBuffer& midiMessages)
 {
     checkInputAndOutput(this, *inputOrderSetting, *outputOrderSetting, false);
     ScopedNoDenormals noDenormals;
 
-    const int nChIn = jmin(buffer.getNumChannels(), input.getNumberOfChannels());
-    const int nChOut = jmin(buffer.getNumChannels(), output.getNumberOfChannels());
-    const int nCh = jmin(nChIn, nChOut);
+    const int nChIn = jmin (buffer.getNumChannels(), input.getNumberOfChannels());
+    const int nChOut = jmin (buffer.getNumChannels(), output.getNumberOfChannels());
+    const int nCh = jmin (nChIn, nChOut);
 
     const int L = buffer.getNumSamples();
 
+    float weights[64];
+    calculateWeights (weights, nChIn, nChOut);
+
+    // apply weights;
+    for (int ch = 0; ch < nCh; ++ch)
+    {
+        if (weights[ch] != previousWeights[ch])
+        {
+            buffer.applyGainRamp(ch, 0, L, previousWeights[ch], weights[ch]);
+            previousWeights[ch] = weights[ch];
+        }
+        else if (weights[ch] != 1.0f)
+        {
+            FloatVectorOperations::multiply (buffer.getWritePointer(ch), weights[ch], L);
+        }
+    }
+
+    // clear not used channels
+    for (int ch = nCh; ch < buffer.getNumChannels(); ++ch)
+    {
+        buffer.clear (ch, 0, L);
+        previousWeights[ch] = 0.0f;
+    }
+
+}
+
+void ToolBoxAudioProcessor::calculateWeights (float *weights, const int nChannelsIn, const int nChannelsOut)
+{
+    const int nCh = jmin (nChannelsIn, nChannelsOut);
     const int orderIn = input.getOrder();
     const int orderOut = output.getOrder();
 
-    float weights[64];
-    FloatVectorOperations::fill(weights, 1.0f, nCh);
+    FloatVectorOperations::fill (weights, Decibels::decibelsToGain (*gain), nCh);
 
     // create mask for all flips
     if (doFlipX || doFlipY || doFlipZ)
@@ -204,52 +191,34 @@ void ToolBoxAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBuffer&
     // lower order ambisonics weighting
     if (orderIn < orderOut)
     {
-        const int weightType = roundToInt(*loaWeights);
+        const int weightType = roundToInt (*loaWeights);
         if (weightType == 1) // maxrE
         {
-            FloatVectorOperations::multiply(weights, getMaxRELUT(orderIn), nChIn);
-            const float* deWeights = getMaxRELUT(orderOut);
-            for (int i = 0; i < nChIn; ++i)
+            FloatVectorOperations::multiply (weights, getMaxRELUT (orderIn), nChannelsIn);
+            const float* deWeights = getMaxRELUT (orderOut);
+            for (int i = 0; i < nChannelsIn; ++i)
                 weights[i] /= deWeights[i];
         }
         else if (weightType == 2) // inPhase
         {
-            FloatVectorOperations::multiply(weights, getInPhaseLUT(orderIn), nChIn);
-            const float* deWeights = getInPhaseLUT(orderOut);
-            for (int i = 0; i < nChIn; ++i)
+            FloatVectorOperations::multiply (weights, getInPhaseLUT (orderIn), nChannelsIn);
+            const float* deWeights = getInPhaseLUT (orderOut);
+            for (int i = 0; i < nChannelsIn; ++i)
                 weights[i] /= deWeights[i];
         }
     }
 
 
     // normalization
-    bool inSN3D = *useSn3dInput >= 0.5f;
-    bool outSN3D = *useSn3dOutput >= 0.5f;
+    const bool inSN3D = *useSn3dInput >= 0.5f;
+    const bool outSN3D = *useSn3dOutput >= 0.5f;
     if (inSN3D != outSN3D)
     {
         if (inSN3D)
-            FloatVectorOperations::multiply(weights, sn3d2n3d, nCh);
+            FloatVectorOperations::multiply (weights, sn3d2n3d, nCh);
         else
-            FloatVectorOperations::multiply(weights, n3d2sn3d, nCh);
+            FloatVectorOperations::multiply (weights, n3d2sn3d, nCh);
     }
-
-
-
-    // apply weights;
-    for (int ch = 0; ch < nCh; ++ch)
-    {
-        if (weights[ch] != 1.0f)
-        {
-            FloatVectorOperations::multiply(buffer.getWritePointer(ch), weights[ch], L);
-        }
-    }
-
-    // clear not used channels
-    for (int ch = nCh; ch < buffer.getNumChannels(); ++ch)
-        buffer.clear(ch, 0, L);
-
-
-
 }
 
 //==============================================================================
@@ -267,7 +236,10 @@ AudioProcessorEditor* ToolBoxAudioProcessor::createEditor()
 void ToolBoxAudioProcessor::getStateInformation (MemoryBlock &destData)
 {
     auto state = parameters.copyState();
-    state.setProperty ("OSCPort", var(oscReceiver.getPortNumber()), nullptr);
+
+    auto oscConfig = state.getOrCreateChildWithName ("OSCConfig", nullptr);
+    oscConfig.copyPropertiesFrom (oscParameterInterface.getConfig(), nullptr);
+
     std::unique_ptr<XmlElement> xml (state.createXml());
     copyXmlToBinary (*xml, destData);
 }
@@ -279,10 +251,15 @@ void ToolBoxAudioProcessor::setStateInformation (const void *data, int sizeInByt
         if (xmlState->hasTagName (parameters.state.getType()))
         {
             parameters.replaceState (ValueTree::fromXml (*xmlState));
-            if (parameters.state.hasProperty ("OSCPort"))
+            if (parameters.state.hasProperty ("OSCPort")) // legacy
             {
-                oscReceiver.connect (parameters.state.getProperty ("OSCPort", var (-1)));
+                oscParameterInterface.getOSCReceiver().connect (parameters.state.getProperty ("OSCPort", var (-1)));
+                parameters.state.removeProperty ("OSCPort", nullptr);
             }
+
+            auto oscConfig = parameters.state.getChildWithName ("OSCConfig");
+            if (oscConfig.isValid())
+                oscParameterInterface.setConfig (oscConfig);
         }
 }
 
@@ -309,50 +286,13 @@ void ToolBoxAudioProcessor::updateBuffers()
 }
 
 //==============================================================================
-pointer_sized_int ToolBoxAudioProcessor::handleVstPluginCanDo (int32 index,
-                                                                     pointer_sized_int value, void* ptr, float opt)
-{
-    auto text = (const char*) ptr;
-    auto matches = [=](const char* s) { return strcmp (text, s) == 0; };
-
-    if (matches ("wantsChannelCountNotifications"))
-        return 1;
-    return 0;
-}
-
-//==============================================================================
-void ToolBoxAudioProcessor::oscMessageReceived (const OSCMessage &message)
-{
-    String prefix ("/" + String(JucePlugin_Name));
-    if (! message.getAddressPattern().toString().startsWith (prefix))
-        return;
-
-    OSCMessage msg (message);
-    msg.setAddressPattern (message.getAddressPattern().toString().substring(String(JucePlugin_Name).length() + 1));
-
-    oscParams.processOSCMessage (msg);
-}
-
-void ToolBoxAudioProcessor::oscBundleReceived (const OSCBundle &bundle)
-{
-    for (int i = 0; i < bundle.size(); ++i)
-    {
-        auto elem = bundle[i];
-        if (elem.isMessage())
-            oscMessageReceived (elem.getMessage());
-        else if (elem.isBundle())
-            oscBundleReceived (elem.getBundle());
-    }
-}
-
-//==============================================================================
-AudioProcessorValueTreeState::ParameterLayout ToolBoxAudioProcessor::createParameterLayout()
+std::vector<std::unique_ptr<RangedAudioParameter>> ToolBoxAudioProcessor::createParameterLayout()
 {
     // add your audio parameters here
     std::vector<std::unique_ptr<RangedAudioParameter>> params;
 
 
-    params.push_back (oscParams.createAndAddParameter ("inputOrderSetting", "Input Ambisonic Order", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("inputOrderSetting", "Input Ambisonic Order", "",
                                      NormalisableRange<float> (0.0f, 8.0f, 1.0f), 0.0f,
                                      [](float value) {
                                          if (value >= 0.5f && value < 1.5f) return "0th";
@@ -366,14 +306,14 @@ AudioProcessorValueTreeState::ParameterLayout ToolBoxAudioProcessor::createParam
                                          else return "Auto";},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("useSn3dInput", "Input Normalization", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("useSn3dInput", "Input Normalization", "",
                                      NormalisableRange<float>(0.0f, 1.0f, 1.0f), 1.0f,
                                      [](float value) {
                                          if (value >= 0.5f) return "SN3D";
                                          else return "N3D";
                                      }, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("outputOrderSetting", "Output Ambisonic Order", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("outputOrderSetting", "Output Ambisonic Order", "",
                                      NormalisableRange<float> (0.0f, 8.0f, 1.0f), 0.0f,
                                      [](float value) {
                                          if (value >= 0.5f && value < 1.5f) return "0th";
@@ -387,38 +327,42 @@ AudioProcessorValueTreeState::ParameterLayout ToolBoxAudioProcessor::createParam
                                          else return "Auto";},
                                      nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("useSn3dOutput", "Output Normalization", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("useSn3dOutput", "Output Normalization", "",
                                      NormalisableRange<float>(0.0f, 1.0f, 1.0f), 1.0f,
                                      [](float value) {
                                          if (value >= 0.5f) return "SN3D";
                                          else return "N3D";
                                      }, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("flipX", "Flip X axis", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("flipX", "Flip X axis", "",
                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0f,
                                      [](float value) {if (value >= 0.5f) return "ON";
                                          else return "OFF";}, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("flipY", "Flip Y axis", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("flipY", "Flip Y axis", "",
                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0f,
                                      [](float value) {if (value >= 0.5f) return "ON";
                                          else return "OFF";}, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("flipZ", "Flip Z axis", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("flipZ", "Flip Z axis", "",
                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0f,
                                      [](float value) {if (value >= 0.5f) return "ON";
                                          else return "OFF";}, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("loaWeights", "Lower Order Ambisonic Weighting", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("loaWeights", "Lower Order Ambisonic Weighting", "",
                                      NormalisableRange<float> (0.0f, 2.0f, 1.0f), 0.0f,
                                      [](float value) {
                                          if (value >= 0.5f && value < 1.5f) return "maxrE";
                                          else if (value >= 1.5f) return "inPhase";
                                          else return "none";},
                                      nullptr));
-    
 
-    return { params.begin(), params.end() };
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("gain", "Gain", "dB",
+                                                                       NormalisableRange<float> (-50.0f, 24.0f, 0.01f), 0.0f,
+                                                                       [](float value) { return String (value, 2); }, nullptr));
+
+
+    return params;
 }
 
 //==============================================================================
