@@ -26,17 +26,18 @@
 
 //==============================================================================
 PluginTemplateAudioProcessor::PluginTemplateAudioProcessor()
-#ifndef JucePlugin_PreferredChannelConfigurations
-     : AudioProcessor (BusesProperties()
-                     #if ! JucePlugin_IsMidiEffect
-                      #if ! JucePlugin_IsSynth
-                       .withInput  ("Input",  AudioChannelSet::discreteChannels (10), true)
-                      #endif
-                       .withOutput ("Output", AudioChannelSet::discreteChannels (64), true)
-                     #endif
-                       ),
+: AudioProcessorBase (
+                      #ifndef JucePlugin_PreferredChannelConfigurations
+                      BusesProperties()
+#if ! JucePlugin_IsMidiEffect
+#if ! JucePlugin_IsSynth
+                      .withInput ("Input",  AudioChannelSet::discreteChannels (10), true)
 #endif
-oscParams (parameters), parameters (*this, nullptr, "PluginTemplate", createParameterLayout())
+                      .withOutput ("Output", AudioChannelSet::discreteChannels (64), true)
+#endif
+                       ,
+#endif
+                       createParameterLayout())
 {
     // get pointers to the parameters
     inputChannelsSetting = parameters.getRawParameterValue ("inputChannelsSetting");
@@ -52,8 +53,6 @@ oscParams (parameters), parameters (*this, nullptr, "PluginTemplate", createPara
     parameters.addParameterListener ("useSN3D", this);
     parameters.addParameterListener ("param1", this);
     parameters.addParameterListener ("param2", this);
-
-    oscReceiver.addListener (this);
 }
 
 PluginTemplateAudioProcessor::~PluginTemplateAudioProcessor()
@@ -61,43 +60,6 @@ PluginTemplateAudioProcessor::~PluginTemplateAudioProcessor()
 }
 
 //==============================================================================
-const String PluginTemplateAudioProcessor::getName() const
-{
-    return JucePlugin_Name;
-}
-
-bool PluginTemplateAudioProcessor::acceptsMidi() const
-{
-   #if JucePlugin_WantsMidiInput
-    return true;
-   #else
-    return false;
-   #endif
-}
-
-bool PluginTemplateAudioProcessor::producesMidi() const
-{
-   #if JucePlugin_ProducesMidiOutput
-    return true;
-   #else
-    return false;
-   #endif
-}
-
-bool PluginTemplateAudioProcessor::isMidiEffect() const
-{
-   #if JucePlugin_IsMidiEffect
-    return true;
-   #else
-    return false;
-   #endif
-}
-
-double PluginTemplateAudioProcessor::getTailLengthSeconds() const
-{
-    return 0.0;
-}
-
 int PluginTemplateAudioProcessor::getNumPrograms()
 {
     return 1;   // NB: some hosts don't cope very well if you tell them there are 0 programs,
@@ -111,27 +73,27 @@ int PluginTemplateAudioProcessor::getCurrentProgram()
 
 void PluginTemplateAudioProcessor::setCurrentProgram (int index)
 {
+    ignoreUnused (index);
 }
 
 const String PluginTemplateAudioProcessor::getProgramName (int index)
 {
+    ignoreUnused (index);
     return {};
 }
 
 void PluginTemplateAudioProcessor::changeProgramName (int index, const String& newName)
 {
+    ignoreUnused (index, newName);
 }
 
 //==============================================================================
 void PluginTemplateAudioProcessor::prepareToPlay (double sampleRate, int samplesPerBlock)
 {
-    checkInputAndOutput (this, *inputChannelsSetting, *outputOrderSetting, true);
-
+    checkInputAndOutput (this, static_cast<int> (*inputChannelsSetting), static_cast<int> (*outputOrderSetting), true);
     // Use this method as the place to do any pre-playback
     // initialisation that you need..
-
-
-
+    ignoreUnused (sampleRate, samplesPerBlock);
 }
 
 void PluginTemplateAudioProcessor::releaseResources()
@@ -140,16 +102,9 @@ void PluginTemplateAudioProcessor::releaseResources()
     // spare memory, etc.
 }
 
-#ifndef JucePlugin_PreferredChannelConfigurations
-bool PluginTemplateAudioProcessor::isBusesLayoutSupported (const BusesLayout& layouts) const
+void PluginTemplateAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBuffer&)
 {
-    return true;
-}
-#endif
-
-void PluginTemplateAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBuffer& midiMessages)
-{
-    checkInputAndOutput (this, *inputChannelsSetting, *outputOrderSetting, false);
+    checkInputAndOutput (this, static_cast<int> (*inputChannelsSetting), static_cast<int> (*outputOrderSetting), false);
     ScopedNoDenormals noDenormals;
 
     const int totalNumInputChannels  = getTotalNumInputChannels();
@@ -169,7 +124,7 @@ void PluginTemplateAudioProcessor::processBlock (AudioSampleBuffer& buffer, Midi
     for (int channel = 0; channel < totalNumInputChannels; ++channel)
     {
         float* channelData = buffer.getWritePointer (channel);
-
+        ignoreUnused (channelData);
         // ..do something to the data...
     }
 }
@@ -188,13 +143,13 @@ AudioProcessorEditor* PluginTemplateAudioProcessor::createEditor()
 //==============================================================================
 void PluginTemplateAudioProcessor::getStateInformation (MemoryBlock& destData)
 {
-    // You should use this method to store your parameters in the memory block.
-    // You could do that either as raw data, or use the XML or ValueTree classes
-    // as intermediaries to make it easy to save and load complex data.
-    auto state = parameters.copyState();
-    state.setProperty ("OSCPort", var (oscReceiver.getPortNumber()), nullptr);
-    std::unique_ptr<XmlElement> xml (state.createXml());
-    copyXmlToBinary (*xml, destData);
+  auto state = parameters.copyState();
+
+  auto oscConfig = state.getOrCreateChildWithName ("OSCConfig", nullptr);
+  oscConfig.copyPropertiesFrom (oscParameterInterface.getConfig(), nullptr);
+
+  std::unique_ptr<XmlElement> xml (state.createXml());
+  copyXmlToBinary (*xml, destData);
 }
 
 
@@ -207,10 +162,15 @@ void PluginTemplateAudioProcessor::setStateInformation (const void* data, int si
         if (xmlState->hasTagName (parameters.state.getType()))
         {
             parameters.replaceState (ValueTree::fromXml (*xmlState));
-            if (parameters.state.hasProperty ("OSCPort"))
+            if (parameters.state.hasProperty ("OSCPort")) // legacy
             {
-                oscReceiver.connect (parameters.state.getProperty ("OSCPort", var (-1)));
+                oscParameterInterface.getOSCReceiver().connect (parameters.state.getProperty ("OSCPort", var (-1)));
+                parameters.state.removeProperty ("OSCPort", nullptr);
             }
+
+            auto oscConfig = parameters.state.getChildWithName ("OSCConfig");
+            if (oscConfig.isValid())
+                oscParameterInterface.setConfig (oscConfig);
         }
 }
 
@@ -229,54 +189,18 @@ void PluginTemplateAudioProcessor::updateBuffers()
     DBG ("IOHelper: output size: " << output.getSize());
 }
 
-//==============================================================================
-pointer_sized_int PluginTemplateAudioProcessor::handleVstPluginCanDo (int32 index,
-                                                                     pointer_sized_int value, void* ptr, float opt)
-{
-    auto text = (const char*) ptr;
-    auto matches = [=](const char* s) { return strcmp (text, s) == 0; };
-
-    if (matches ("wantsChannelCountNotifications"))
-        return 1;
-    return 0;
-}
 
 //==============================================================================
-void PluginTemplateAudioProcessor::oscMessageReceived (const OSCMessage &message)
-{
-    String prefix ("/" + String (JucePlugin_Name));
-    if (! message.getAddressPattern().toString().startsWith (prefix))
-        return;
-
-    OSCMessage msg (message);
-    msg.setAddressPattern (message.getAddressPattern().toString().substring (String (JucePlugin_Name).length() + 1));
-
-    oscParams.processOSCMessage (msg);
-}
-
-void PluginTemplateAudioProcessor::oscBundleReceived (const OSCBundle &bundle)
-{
-    for (int i = 0; i < bundle.size(); ++i)
-    {
-        auto elem = bundle[i];
-        if (elem.isMessage())
-            oscMessageReceived (elem.getMessage());
-        else if (elem.isBundle())
-            oscBundleReceived (elem.getBundle());
-    }
-}
-
-//==============================================================================
-AudioProcessorValueTreeState::ParameterLayout PluginTemplateAudioProcessor::createParameterLayout()
+std::vector<std::unique_ptr<RangedAudioParameter>> PluginTemplateAudioProcessor::createParameterLayout()
 {
     // add your audio parameters here
     std::vector<std::unique_ptr<RangedAudioParameter>> params;
 
-    params.push_back (oscParams.createAndAddParameter ("inputChannelsSetting", "Number of input channels ", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("inputChannelsSetting", "Number of input channels ", "",
                                                        NormalisableRange<float> (0.0f, 10.0f, 1.0f), 0.0f,
                                                        [](float value) {return value < 0.5f ? "Auto" : String (value);}, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("outputOrderSetting", "Ambisonic Order", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("outputOrderSetting", "Ambisonic Order", "",
                                                        NormalisableRange<float> (0.0f, 8.0f, 1.0f), 0.0f,
                                                        [](float value) {
                                                            if (value >= 0.5f && value < 1.5f) return "0th";
@@ -290,23 +214,22 @@ AudioProcessorValueTreeState::ParameterLayout PluginTemplateAudioProcessor::crea
                                                            else return "Auto";},
                                                        nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("useSN3D", "Normalization", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("useSN3D", "Normalization", "",
                                                        NormalisableRange<float>(0.0f, 1.0f, 1.0f), 1.0f,
                                                        [](float value) {
                                                            if (value >= 0.5f) return "SN3D";
                                                            else return "N3D";
                                                        }, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("param1", "Parameter 1", "",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("param1", "Parameter 1", "",
                                                        NormalisableRange<float> (-10.0f, 10.0f, 0.1f), 0.0,
                                                        [](float value) {return String (value);}, nullptr));
 
-    params.push_back (oscParams.createAndAddParameter ("param2", "Parameter 2", "dB",
+    params.push_back (OSCParameterInterface::createParameterTheOldWay ("param2", "Parameter 2", "dB",
                                                        NormalisableRange<float> (-50.0f, 0.0f, 0.1f), -10.0,
                                                        [](float value) {return String (value, 1);}, nullptr));
 
-
-    return { params.begin(), params.end() };
+    return params;
 }
 
 
