@@ -23,53 +23,30 @@
 #pragma once
 
 #include "../JuceLibraryCode/JuceHeader.h"
-#include "../../resources/IOHelper.h"
-
-// ===== OSC ====
-#include "../../resources/OSCParameterInterface.h"
-#include "../../resources/OSCReceiverPlus.h"
+#include "../../resources/AudioProcessorBase.h"
 
+#define ProcessorClass PluginTemplateAudioProcessor
 
 //==============================================================================
-/**
- Use the IOHelper to detect which amount of channels or which Ambisonic order is possible with the current bus layout.
- The available IOTypes are:
-    - AudioChannels<maxChannelCount>
-    - Ambisonics<maxOrder> (can also be used for directivity signals)
- You can leave `maxChannelCount` and `maxOrder` empty for default values (64 channels and 7th order)
-*/
-class PluginTemplateAudioProcessor  : public AudioProcessor,
-                                        public AudioProcessorValueTreeState::Listener,
-                                        public IOHelper<IOTypes::AudioChannels<10>, IOTypes::Ambisonics<7>>,
-                                        public VSTCallbackHandler,
-                                        private OSCReceiver::Listener<OSCReceiver::RealtimeCallback>
+class PluginTemplateAudioProcessor  :  public AudioProcessorBase<IOTypes::AudioChannels<10>, IOTypes::Ambisonics<7>>
 {
 public:
+    constexpr static int numberOfInputChannels = 10;
+    constexpr static int numberOfOutputChannels = 64;
     //==============================================================================
     PluginTemplateAudioProcessor();
-    ~PluginTemplateAudioProcessor();
+    ~PluginTemplateAudioProcessor() override;
 
     //==============================================================================
     void prepareToPlay (double sampleRate, int samplesPerBlock) override;
     void releaseResources() override;
 
-   #ifndef JucePlugin_PreferredChannelConfigurations
-    bool isBusesLayoutSupported (const BusesLayout& layouts) const override;
-   #endif
-
     void processBlock (AudioSampleBuffer&, MidiBuffer&) override;
 
     //==============================================================================
     AudioProcessorEditor* createEditor() override;
     bool hasEditor() const override;
 
-    //==============================================================================
-    const String getName() const override;
-
-    bool acceptsMidi() const override;
-    bool producesMidi() const override;
-    bool isMidiEffect () const override;
-    double getTailLengthSeconds() const override;
 
     //==============================================================================
     int getNumPrograms() override;
@@ -86,32 +63,14 @@ public:
     void parameterChanged (const String &parameterID, float newValue) override;
     void updateBuffers() override; // use this to implement a buffer update method
 
-    //======== PluginCanDo =========================================================
-    pointer_sized_int handleVstManufacturerSpecific (int32 index, pointer_sized_int value,
-                                                     void* ptr, float opt) override { return 0; };
-
-    pointer_sized_int handleVstPluginCanDo (int32 index, pointer_sized_int value,
-                                            void* ptr, float opt) override;
-    //==============================================================================
-
-
-    //======== OSC =================================================================
-    void oscMessageReceived (const OSCMessage &message) override;
-    void oscBundleReceived (const OSCBundle &bundle) override;
-    OSCReceiverPlus& getOSCReceiver () { return oscReceiver; }
-    //==============================================================================
 
     //======= Parameters ===========================================================
-    AudioProcessorValueTreeState::ParameterLayout createParameterLayout();
+    std::vector<std::unique_ptr<RangedAudioParameter>> createParameterLayout();
     //==============================================================================
 
 
 private:
-    // ====== parameters
-    OSCParameterInterface oscParams;
-    OSCReceiverPlus oscReceiver;
-    AudioProcessorValueTreeState parameters;
-    
+    //==============================================================================
     // list of used audio parameters
     float *inputChannelsSetting, *outputOrderSetting, *useSN3D, *param1, *param2;
 
