@@ -322,7 +322,12 @@ public:
 
     void setSampleRate (const double newSampleRate)
     {
-        sampleRate = newSampleRate;
+        if (newSampleRate == 0)
+            sampleRate = 48000.0;
+        else
+            sampleRate = newSampleRate;
+
+        repaint();
     }
 
     void setOverallGain (float newGain)
@@ -494,7 +499,7 @@ private:
 
     double sampleRate;
 
-    int activeElem = 0;
+    int activeElem = -1;
 
     float dyn, zero, scale;
 
