@@ -41,9 +41,17 @@ public:
     void paint (Graphics& g) override
     {
         auto bounds = getLocalBounds();
-
-        const float boxSize = bounds.getWidth() * 0.8f;
-        Rectangle<float> buttonArea((bounds.getWidth() - boxSize) * 0.5f, (bounds.getHeight() - boxSize) * 0.5f, boxSize, boxSize);
+        Rectangle<float> buttonArea;
+        if (isCircularShape)
+        {
+            const float boxSize = bounds.getWidth() >= bounds.getHeight() ? bounds.getHeight() * 0.8f : bounds.getWidth() * 0.8f;
+            buttonArea = Rectangle<float> ((bounds.getWidth() - boxSize) * 0.5f, (bounds.getHeight() - boxSize) * 0.5f, boxSize, boxSize);
+        }
+        else
+        {
+            buttonArea = Rectangle<float> (bounds.getX(), bounds.getY(), bounds.getWidth(), bounds.getHeight());
+            buttonArea.reduce (0.5f, 0.4f);
+        }
 
         const bool isButtonDown = isMouseButtonDown();
         const bool isMouseOverButton = isMouseOver();
@@ -55,23 +63,23 @@ public:
         else if (isMouseOverButton)
             buttonArea.reduce(0.4f, 0.4f);
 
-        g.setColour(findColour(ToggleButton::tickColourId).withMultipliedAlpha(ticked ? 1.0f : isMouseOverButton ? 0.7f : 0.5f) );
+        g.setColour (findColour (ToggleButton::tickColourId).withMultipliedAlpha (ticked ? 1.0f : isMouseOverButton ? 0.7f : 0.5f) );
 
-        g.drawEllipse(buttonArea, 1.0f);
+        isCircularShape == true ? g.drawEllipse (buttonArea, 1.0f) : g.drawRoundedRectangle (buttonArea, 10.0f, 1.0f);
 
-        buttonArea.reduce(1.5f, 1.5f);
-        g.setColour(findColour(ToggleButton::tickColourId).withMultipliedAlpha(ticked ? 1.0f : isMouseOverButton ? 0.5f : 0.2f));
+        buttonArea.reduce (1.5f, 1.5f);
+        g.setColour (findColour (ToggleButton::tickColourId).withMultipliedAlpha (ticked ? 1.0f : isMouseOverButton ? 0.5f : 0.2f));
 
-        g.fillEllipse(buttonArea);
+        isCircularShape == true ? g.fillEllipse (buttonArea) : g.fillRoundedRectangle (buttonArea, 10.0f);
 
-        
-        g.setFont(getLookAndFeel().getTypefaceForFont (Font(13.0f, 1)));
-        g.setFont (buttonArea.getHeight());
+
+        g.setFont (getLookAndFeel().getTypefaceForFont (Font (13.0f, 1)));
+        g.setFont (buttonArea.getHeight() * scaleFontSize);
         g.setColour (findColour (getToggleState() ? TextButton::textColourOnId
                                  : TextButton::textColourOffId)
                      .withMultipliedAlpha (isEnabled() ? 1.0f : 0.5f));
 
-        g.setColour(ticked ? findColour(ResizableWindow::backgroundColourId) :  findColour(ToggleButton::tickColourId).withMultipliedAlpha(isMouseOverButton ? 0.7f : 0.5f) );
+        g.setColour (ticked ? findColour (ResizableWindow::backgroundColourId) :  findColour (ToggleButton::tickColourId).withMultipliedAlpha (isMouseOverButton ? 0.7f : 0.5f));
 
         g.drawText(getButtonText(), getLocalBounds(), Justification::centred);
     }
@@ -80,6 +88,19 @@ public:
     {
     }
 
+    void setScaleFontSize (const float newScale)
+    {
+        scaleFontSize = newScale;
+    }
+
+    void setCircularShape (bool shouldBeCircularShape)
+    {
+        isCircularShape = shouldBeCircularShape;
+    }
+
 private:
+    bool isCircularShape = true;
+    float scaleFontSize = 1.0f;
+
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (RoundButton)
 };
